@@ -18,8 +18,11 @@
  *  limitations under the License.
  */
 
-#ifndef FLB_END_H
-#define FLB_END_H
+#ifndef FLB_ENV_H
+#define FLB_ENV_H
+
+#include <fluent-bit/flb_info.h>
+#include <fluent-bit/flb_sds.h>
 
 #define FLB_ENV_SIZE 64
 
@@ -31,6 +34,6 @@ struct flb_env *flb_env_create();
 void flb_env_destroy(struct flb_env *env);
 int flb_env_set(struct flb_env *env, const char *key, const char *val);
 const char *flb_env_get(struct flb_env *env, const char *key);
-char *flb_env_var_translate(struct flb_env *env, const char *value);
+flb_sds_t flb_env_var_translate(struct flb_env *env, const char *value);
 
 #endif
