@@ -237,6 +237,16 @@ static int mk_rconf_read(struct mk_rconf *conf, const char *path)
                 buf[--len] = 0;
             }
         }
+        else {
+            /*
+             * If we don't find a break line, validate if we got an EOF or not. No EOF
+             * means that the incoming string is not finished so we must raise an
+             * exception.
+             */
+            if (!feof(f)) {
+                mk_config_error(path, line, "Length of content has exceeded limit");
+            }
+        }
 
         /* Line number */
         line++;
