@@ -36,6 +36,13 @@
 #include "kube_meta.h"
 #include "kube_property.h"
 
+#define FLB_KUBE_META_CONTAINER_STATUSES_KEY "containerStatuses"
+#define FLB_KUBE_META_CONTAINER_STATUSES_KEY_LEN \
+    (sizeof(FLB_KUBE_META_CONTAINER_STATUSES_KEY) - 1)
+#define FLB_KUBE_META_INIT_CONTAINER_STATUSES_KEY "initContainerStatuses"
+#define FLB_KUBE_META_INIT_CONTAINER_STATUSES_KEY_LEN \
+    (sizeof(FLB_KUBE_META_INIT_CONTAINER_STATUSES_KEY) - 1)
+
 static int file_to_buffer(const char *path,
                           char **out_buf, size_t *out_size)
 {
@@ -152,24 +159,31 @@ static int get_api_server_info(struct flb_kube *ctx,
     struct flb_http_client *c;
     struct flb_upstream_conn *u_conn;
 
+    *out_buf = NULL;
+    *out_size = 0;
+
     /*
-     * If a file exists called namespace-podname.meta, load it and use it.
+     * If a file exists called namespace_podname.meta, load it and use it.
      * If not, fall back to API. This is primarily for diagnostic purposes,
      * e.g. debugging new parsers.
      */
     if (ctx->meta_preload_cache_dir && namespace && podname) {
-        int fd;
+        int fd = -1;
         char *payload = NULL;
         size_t payload_size = 0;
         struct stat sb;
 
-        ret = snprintf(uri, sizeof(uri) - 1, "%s/%s-%s.meta", ctx->meta_preload_cache_dir, namespace, podname);
+        ret = snprintf(uri, sizeof(uri) - 1, "%s/%s_%s.meta",
+                       ctx->meta_preload_cache_dir, namespace, podname);
         if (ret > 0) {
             fd = open(uri, O_RDONLY, 0);
-            if (fd > 0) {
+            if (fd != -1) {
                 if (fstat(fd, &sb) == 0) {
                     payload = flb_malloc(sb.st_size);
-                    if (payload) {
+                    if (!payload) {
+                        flb_errno();
+                    }
+                    else {
                         ret = read(fd, payload, sb.st_size);
                         if (ret == sb.st_size) {
                             payload_size = ret;
@@ -179,6 +193,7 @@ static int get_api_server_info(struct flb_kube *ctx,
                 close(fd);
             }
         }
+
         if (payload_size) {
             packed = flb_pack_json(payload, payload_size,
                                    &buf, &size, &root_type);
@@ -258,6 +273,10 @@ static void cb_results(const char *name, const char *value,
 {
     struct flb_kube_meta *meta = data;
 
+    if (vlen == 0) {
+        return;
+    }
+
     if (meta->podname == NULL && strcmp(name, "pod_name") == 0) {
         meta->podname = flb_strndup(value, vlen);
         meta->podname_len = vlen;
@@ -273,29 +292,75 @@ static void cb_results(const char *name, const char *value,
              strcmp(name, "container_name") == 0) {
         meta->container_name = flb_strndup(value, vlen);
         meta->container_name_len = vlen;
-        meta->skip++;
+        meta->fields++;
     }
     else if (meta->docker_id == NULL &&
              strcmp(name, "docker_id") == 0) {
         meta->docker_id = flb_strndup(value, vlen);
         meta->docker_id_len = vlen;
-        meta->skip++;
+        meta->fields++;
     }
     else if (meta->container_hash == NULL &&
              strcmp(name, "container_hash") == 0) {
         meta->container_hash = flb_strndup(value, vlen);
         meta->container_hash_len = vlen;
-        meta->skip++;
+        meta->fields++;
     }
 
     return;
 }
 
+static int extract_hash(const char * im, int sz, const char ** out, int * outsz)
+{
+    char * colon = NULL;
+    char * slash = NULL;
+
+    *out = NULL;
+    *outsz = 0;
+
+    if (sz <= 1) {
+        return -1;
+    }
+
+    colon = memrchr(im, ':', sz);
+
+    if (colon == NULL) {
+        return -1;
+    } else {
+        slash = colon;
+        while ((im + sz - slash + 1) > 0 && *(slash + 1) == '/') {
+            slash++;
+        }
+        if (slash == colon) {
+            slash = NULL;
+        }
+    }
+
+    if (slash == NULL && (im + sz - colon) > 0) {
+        *out = colon + 1;
+    }
+
+    if (slash != NULL) {
+        if ((colon - slash) < 0 && (im + sz - slash) > 0) {
+            *out = slash + 1;
+        } else if ((colon - slash) > 0 && (im + sz - colon) > 0) {
+            *out = colon + 1;
+        }
+    }
+
+    if (*out) {
+        *outsz = im + sz - *out;
+        return 0;
+    }
+    return -1;
+}
+
 /*
  * As per Kubernetes Pod spec,
  * https://kubernetes.io/docs/concepts/workloads/pods/pod/, we look
- * for status.containerStatuses.{containerID, imageID} where
- * status.containerStatus.name == our container name
+ * for status.{initContainerStatuses, containerStatuses}.{containerID, imageID}
+ * where status.{initContainerStatuses, containerStatus}.name == our container
+ * name
  * status:
  *   ...
  *   containerStatuses:
@@ -305,26 +370,32 @@ static void cb_results(const char *name, const char *value,
  *     ...
  *     name: nginx-ingress-microk8s
 */
-#define CONTAINER_STATUSES "containerStatuses"
 static void extract_container_hash(struct flb_kube_meta *meta,
                                    msgpack_object status)
 {
     int i;
-    msgpack_object k, v;
+    int name_found = FLB_FALSE;
     int docker_id_len = 0;
     int container_hash_len = 0;
     const char *container_hash;
     const char *docker_id;
-    int name_found = FLB_FALSE;
+    msgpack_object k, v;
+    const char *tmp;
+    int tmp_len = 0;
+
     /* Process status/containerStatus map for docker_id, container_hash */
     for (i = 0;
          (meta->docker_id_len == 0 || meta->container_hash_len == 0) &&
          i < status.via.map.size; i++) {
         k = status.via.map.ptr[i].key;
-        if (k.via.str.size == sizeof(CONTAINER_STATUSES)-1 &&
-            strncmp(k.via.str.ptr,
-                    CONTAINER_STATUSES,
-                    sizeof(CONTAINER_STATUSES)-1) == 0) {
+        if ((k.via.str.size == FLB_KUBE_META_CONTAINER_STATUSES_KEY_LEN &&
+             strncmp(k.via.str.ptr,
+                     FLB_KUBE_META_CONTAINER_STATUSES_KEY,
+                     FLB_KUBE_META_CONTAINER_STATUSES_KEY_LEN) == 0) ||
+            (k.via.str.size == FLB_KUBE_META_INIT_CONTAINER_STATUSES_KEY_LEN &&
+             strncmp(k.via.str.ptr,
+                     FLB_KUBE_META_INIT_CONTAINER_STATUSES_KEY,
+                     FLB_KUBE_META_INIT_CONTAINER_STATUSES_KEY_LEN) == 0)) {
             int j;
             v = status.via.map.ptr[i].val;
             for (j = 0;
@@ -341,7 +412,7 @@ static void extract_container_hash(struct flb_kube_meta *meta,
                      l < k1.via.map.size; l++) {
                     k2 = k1.via.map.ptr[l].key;
                     v2 = k1.via.map.ptr[l].val.via.str;
-                    if (k2.via.str.size == sizeof("name") -1 &&
+                    if (k2.via.str.size == sizeof("name") - 1 &&
                         !strncmp(k2.via.str.ptr, "name", k2.via.str.size)) {
                         if (v2.size == meta->container_name_len &&
                             !strncmp(v2.ptr,
@@ -349,34 +420,43 @@ static void extract_container_hash(struct flb_kube_meta *meta,
                                      meta->container_name_len)) {
                             name_found = FLB_TRUE;
                         }
+                        else {
+                            break;
+                        }
                     }
-                    if (k2.via.str.size == sizeof("containerID") - 1 &&
+                    else if (k2.via.str.size == sizeof("containerID") - 1 &&
                         !strncmp(k2.via.str.ptr,
                                  "containerID",
                                  k2.via.str.size)) {
-                        docker_id = v2.ptr;
-                        docker_id_len = v2.size;
+                        if (extract_hash(v2.ptr, v2.size, &tmp, &tmp_len) == 0) {
+                            docker_id = tmp;
+                            docker_id_len = tmp_len;
+                        }
                     }
                     else if (k2.via.str.size == sizeof("imageID") - 1 &&
-                              !strncmp(v2.ptr,
+                              !strncmp(k2.via.str.ptr,
                                        "imageID",
-                                      v2.size)) {
-                        container_hash = v2.ptr;
-                        container_hash_len = v2.size;
+                                       k2.via.str.size)) {
+                        if (extract_hash(v2.ptr, v2.size, &tmp, &tmp_len) == 0) {
+                            container_hash = tmp;
+                            container_hash_len = tmp_len;
+                        }
                     }
                 }
+
                 if (name_found) {
                     if (container_hash_len && !meta->container_hash_len) {
                         meta->container_hash_len = container_hash_len;
                         meta->container_hash = flb_strndup(container_hash,
                                                            container_hash_len);
-                        meta->skip++;
+                        meta->fields++;
                     }
                     if (docker_id_len && !meta->docker_id_len) {
                         meta->docker_id_len = docker_id_len;
                         meta->docker_id = flb_strndup(docker_id, docker_id_len);
-                        meta->skip++;
+                        meta->fields++;
                     }
+                    return;
                 }
             }
         }
@@ -389,7 +469,7 @@ static int merge_meta(struct flb_kube_meta *meta, struct flb_kube *ctx,
 {
     int i;
     int ret;
-    int map_size;
+    int map_size = 0;
     int meta_found = FLB_FALSE;
     int spec_found = FLB_FALSE;
     int status_found = FLB_FALSE;
@@ -429,108 +509,101 @@ static int merge_meta(struct flb_kube_meta *meta, struct flb_kube *ctx,
     msgpack_sbuffer_init(&mp_sbuf);
     msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
 
-    /* Set map size: current + pod_id, labels and annotations */
-    map_size = meta->fields;
-
     /* Iterate API server msgpack and lookup specific fields */
-    off = 0;
-    msgpack_unpacked_init(&api_result);
-    ret = msgpack_unpack_next(&api_result, api_buf, api_size, &off);
-    if (ret != MSGPACK_UNPACK_SUCCESS) {
-        msgpack_sbuffer_destroy(&mp_sbuf);
-        msgpack_unpacked_destroy(&api_result);
-        return -1;
-    }
+    if (api_buf != NULL) {
+        msgpack_unpacked_init(&api_result);
+        ret = msgpack_unpack_next(&api_result, api_buf, api_size, &off);
+        if (ret == MSGPACK_UNPACK_SUCCESS) {
+            api_map = api_result.data;
 
-    api_map = api_result.data;
-
-    /* At this point map points to the ROOT map, eg:
-     *
-     * {
-     *  "kind": "Pod",
-     *  "apiVersion": "v1",
-     *  "metadata": {
-     *    "name": "fluent-bit-rz47v",
-     *    "generateName": "fluent-bit-",
-     *    "namespace": "kube-system",
-     *    "selfLink": "/api/v1/namespaces/kube-system/pods/fluent-bit-rz47v",
-     *  ....
-     * }
-     *
-     * We are interested into the 'metadata' map value.
-     * We are also interested in the spec.nodeName.
-     * We are also interested in the status.containerStatuses.
-     */
-    for (i = 0; !(meta_found && spec_found && status_found) &&
-                i < api_map.via.map.size; i++) {
-	k = api_map.via.map.ptr[i].key;
-        if (k.via.str.size == 8 && !strncmp(k.via.str.ptr, "metadata", 8)) {
-            meta_val = api_map.via.map.ptr[i].val;
-            meta_found = FLB_TRUE;
-        }
-	else if (k.via.str.size == 4 && !strncmp(k.via.str.ptr, "spec", 4)) {
-           spec_val = api_map.via.map.ptr[i].val;
-           spec_found = FLB_TRUE;
-        }
-	else if (k.via.str.size == 6 && !strncmp(k.via.str.ptr, "status", 6)) {
-           status_val = api_map.via.map.ptr[i].val;
-           status_found = FLB_TRUE;
-	}
-    }
-
-    if (meta_found == FLB_FALSE) {
-        msgpack_unpacked_destroy(&api_result);
-        msgpack_sbuffer_destroy(&mp_sbuf);
-        return -1;
-    }
-
-    /* Process metadata map value */
-    msgpack_unpacked_init(&meta_result);
-    for (i = 0; i < meta_val.via.map.size; i++) {
-        k = meta_val.via.map.ptr[i].key;
-
-        const char *ptr = k.via.str.ptr;
-        size_t size = k.via.str.size;
-
-        if (size == 3 && strncmp(ptr, "uid", 3) == 0) {
-            have_uid = i;
-            map_size++;
-        }
-        else if (size == 6 && strncmp(ptr, "labels", 6) == 0) {
-            have_labels = i;
-            if (ctx->labels == FLB_TRUE) {
-                map_size++;
+            /* At this point map points to the ROOT map, eg:
+             *
+             * {
+             *  "kind": "Pod",
+             *  "apiVersion": "v1",
+             *  "metadata": {
+             *    "name": "fluent-bit-rz47v",
+             *    "generateName": "fluent-bit-",
+             *    "namespace": "kube-system",
+             *    "selfLink": "/api/v1/namespaces/kube-system/pods/fluent-bit-rz47v",
+             *  ....
+             * }
+             *
+             * We are interested into the 'metadata' map value.
+             * We are also interested in the spec.nodeName.
+             * We are also interested in the status.containerStatuses.
+             */
+            for (i = 0; !(meta_found && spec_found && status_found) &&
+                        i < api_map.via.map.size; i++) {
+                k = api_map.via.map.ptr[i].key;
+                if (k.via.str.size == 8 && !strncmp(k.via.str.ptr, "metadata", 8)) {
+                    meta_val = api_map.via.map.ptr[i].val;
+                    meta_found = FLB_TRUE;
+                }
+                else if (k.via.str.size == 4 && !strncmp(k.via.str.ptr, "spec", 4)) {
+                   spec_val = api_map.via.map.ptr[i].val;
+                   spec_found = FLB_TRUE;
+                }
+                else if (k.via.str.size == 6 && !strncmp(k.via.str.ptr, "status", 6)) {
+                   status_val = api_map.via.map.ptr[i].val;
+                   status_found = FLB_TRUE;
+                }
             }
-        }
 
-        else if (size == 11 && strncmp(ptr, "annotations", 11) == 0) {
-            have_annotations = i;
-            if (ctx->annotations == FLB_TRUE) {
-                map_size++;
+            if (meta_found == FLB_TRUE) {
+                /* Process metadata map value */
+                msgpack_unpacked_init(&meta_result);
+                for (i = 0; i < meta_val.via.map.size; i++) {
+                    k = meta_val.via.map.ptr[i].key;
+
+                    char *ptr = (char *) k.via.str.ptr;
+                    size_t size = k.via.str.size;
+
+                    if (size == 3 && strncmp(ptr, "uid", 3) == 0) {
+                        have_uid = i;
+                        map_size++;
+                    }
+                    else if (size == 6 && strncmp(ptr, "labels", 6) == 0) {
+                        have_labels = i;
+                        if (ctx->labels == FLB_TRUE) {
+                            map_size++;
+                        }
+                    }
+
+                    else if (size == 11 && strncmp(ptr, "annotations", 11) == 0) {
+                        have_annotations = i;
+                        if (ctx->annotations == FLB_TRUE) {
+                            map_size++;
+                        }
+                    }
+
+                    if (have_uid >= 0 && have_labels >= 0 && have_annotations >= 0) {
+                        break;
+                    }
+                }
             }
-        }
 
-        if (have_uid >= 0 && have_labels >= 0 && have_annotations >= 0) {
-            break;
-        }
-    }
+            /* Process spec map value for nodeName */
+            if (spec_found == FLB_TRUE) {
+                for (i = 0; i < spec_val.via.map.size; i++) {
+                    k = spec_val.via.map.ptr[i].key;
+                    if (k.via.str.size == 8 &&
+                        strncmp(k.via.str.ptr, "nodeName", 8) == 0) {
+                        have_nodename = i;
+                        map_size++;
+                        break;
+                    }
+                }
+            }
 
-    /* Process spec map value for nodeName */
-    if (spec_found == FLB_TRUE) {
-        for (i = 0; i < spec_val.via.map.size; i++) {
-            k = spec_val.via.map.ptr[i].key;
-            if (k.via.str.size == 8 &&
-                strncmp(k.via.str.ptr, "nodeName", 8) == 0) {
-                have_nodename = i;
-                map_size++;
-                break;
+            if ((!meta->container_hash || !meta->docker_id) && status_found) {
+                extract_container_hash(meta, status_val);
             }
         }
     }
 
-    if ((!meta->container_hash || !meta->docker_id) && status_found) {
-        extract_container_hash(meta, status_val);
-    }
+    /* Set map size: current + pod_id, labels and annotations */
+    map_size += meta->fields;
 
     /* Append Regex fields */
     msgpack_pack_map(&mp_pck, map_size);
@@ -580,6 +653,28 @@ static int merge_meta(struct flb_kube_meta *meta, struct flb_kube *ctx,
         msgpack_pack_object(&mp_pck, v);
     }
 
+    if (meta->container_name != NULL) {
+        msgpack_pack_str(&mp_pck, 14);
+        msgpack_pack_str_body(&mp_pck, "container_name", 14);
+        msgpack_pack_str(&mp_pck, meta->container_name_len);
+        msgpack_pack_str_body(&mp_pck, meta->container_name,
+                              meta->container_name_len);
+    }
+    if (meta->docker_id != NULL) {
+        msgpack_pack_str(&mp_pck, 9);
+        msgpack_pack_str_body(&mp_pck, "docker_id", 9);
+        msgpack_pack_str(&mp_pck, meta->docker_id_len);
+        msgpack_pack_str_body(&mp_pck, meta->docker_id,
+                              meta->docker_id_len);
+    }
+    if (meta->container_hash != NULL) {
+        msgpack_pack_str(&mp_pck, 14);
+        msgpack_pack_str_body(&mp_pck, "container_hash", 14);
+        msgpack_pack_str(&mp_pck, meta->container_hash_len);
+        msgpack_pack_str_body(&mp_pck, meta->container_hash,
+                              meta->container_hash_len);
+    }
+
     /* Process configuration suggested through Annotations */
     if (have_annotations >= 0) {
         ann_map = meta_val.via.map.ptr[have_annotations].val;
@@ -613,8 +708,12 @@ static int merge_meta(struct flb_kube_meta *meta, struct flb_kube *ctx,
         flb_free(prop_buf);
     }
 
-    msgpack_unpacked_destroy(&api_result);
-    msgpack_unpacked_destroy(&meta_result);
+    if (api_buf != NULL) {
+        msgpack_unpacked_destroy(&api_result);
+        if (meta_found == FLB_TRUE) {
+            msgpack_unpacked_destroy(&meta_result);
+        }
+    }
 
     /* Set outgoing msgpack buffer */
     *out_buf = mp_sbuf.data;
@@ -696,7 +795,7 @@ static inline int extract_meta(struct flb_kube *ctx,
         kube_tag_len = flb_sds_len(ctx->kube_tag_prefix);
         if (kube_tag_len + 1 >= tag_len) {
             flb_error("[filter_kube] incoming record tag (%s) is shorter "
-                      "than kube_tag_prefix value (%s)",
+                      "than kube_tag_prefix value (%s), skip filter",
                       tag, ctx->kube_tag_prefix);
             return -1;
         }
@@ -765,29 +864,20 @@ static int get_and_merge_meta(struct flb_kube *ctx, struct flb_kube_meta *meta,
     int ret;
     char *api_buf;
     size_t api_size;
-    char *merge_buf;
-    size_t merge_size;
 
-    ret = get_api_server_info(ctx,
-                              meta->namespace, meta->podname,
-                              &api_buf, &api_size);
-    if (ret == -1) {
-        return -1;
-    }
+    get_api_server_info(ctx,
+                        meta->namespace, meta->podname,
+                        &api_buf, &api_size);
 
     ret = merge_meta(meta, ctx,
                      api_buf, api_size,
-                     &merge_buf, &merge_size);
-    flb_free(api_buf);
+                     out_buf, out_size);
 
-    if (ret == -1) {
-        return -1;
+    if (api_buf != NULL) {
+        flb_free(api_buf);
     }
 
-    *out_buf = merge_buf;
-    *out_size = merge_size;
-
-    return 0;
+    return ret;
 }
 
 
@@ -803,6 +893,7 @@ static int flb_kube_network_init(struct flb_kube *ctx, struct flb_config *config
         }
         ctx->tls.context = flb_tls_context_new(ctx->tls_verify,
                                                ctx->tls_debug,
+                                               NULL, /* skip vhost */
                                                ctx->tls_ca_path,
                                                ctx->tls_ca_file,
                                                NULL, NULL, NULL);
@@ -841,36 +932,36 @@ int flb_kube_meta_init(struct flb_kube *ctx, struct flb_config *config)
         return 0;
     }
 
+    /* Init network */
+    flb_kube_network_init(ctx, config);
+
     /* Gather local info */
     ret = get_local_pod_info(ctx);
     if (ret == FLB_TRUE) {
         flb_info("[filter_kube] local POD info OK");
+
+        /* Gather info from API server */
+        flb_info("[filter_kube] testing connectivity with API server...");
+        ret = get_api_server_info(ctx, ctx->namespace, ctx->podname,
+                                  &meta_buf, &meta_size);
+        if (ret == -1) {
+            if (!ctx->podname) {
+                flb_warn("[filter_kube] could not get meta for local POD");
+            }
+            else {
+                flb_warn("[filter_kube] could not get meta for POD %s",
+                         ctx->podname);
+            }
+            return -1;
+        }
+        flb_info("[filter_kube] API server connectivity OK");
+
+        flb_free(meta_buf);
     }
     else {
         flb_info("[filter_kube] not running in a POD");
     }
 
-    /* Init network */
-    flb_kube_network_init(ctx, config);
-
-    /* Gather info from API server */
-    flb_info("[filter_kube] testing connectivity with API server...");
-    ret = get_api_server_info(ctx, ctx->namespace, ctx->podname,
-                              &meta_buf, &meta_size);
-    if (ret == -1) {
-        if (!ctx->podname) {
-            flb_warn("[filter_kube] could not get meta for local POD");
-        }
-        else {
-            flb_warn("[filter_kube] could not get meta for POD %s",
-                     ctx->podname);
-        }
-        return -1;
-    }
-    flb_info("[filter_kube] API server connectivity OK");
-
-    flb_free(meta_buf);
-
     return 0;
 }
 
@@ -924,16 +1015,29 @@ int flb_kube_meta_get(struct flb_kube *ctx,
         return -1;
     }
 
-    /* Check if we have some data associated to the cache key */
-    ret = flb_hash_get(ctx->hash_table,
-                       meta->cache_key, meta->cache_key_len,
-                       &hash_meta_buf, &hash_meta_size);
+    if (ctx->cache == FLB_TRUE) {
+        /* Check if we have some data associated to the cache key */
+        ret = flb_hash_get(ctx->hash_table,
+                           meta->cache_key, meta->cache_key_len,
+                           &hash_meta_buf, &hash_meta_size);
+    }
+    else {
+        /*
+         * Metadata cache is disabled, force metadata retrieval. Note that this
+         * mechanism is only used for testing purposes, we don't enforce the check
+         * again (hash table will be populated anyways, not relevant).
+         */
+        ret = -1;
+    }
+
     if (ret == -1) {
         /* Retrieve API server meta and merge with local meta */
         ret = get_and_merge_meta(ctx, meta,
                                  &tmp_hash_meta_buf, &hash_meta_size);
         if (ret == -1) {
-            return -1;
+            *out_buf = NULL;
+            *out_size = 0;
+            return 0;
         }
 
         id = flb_hash_add(ctx->hash_table,
