@@ -24,6 +24,11 @@
 #include "kube_meta.h"
 #include "kube_props.h"
 
+#define FLB_KUBE_PROP_NO_STREAM 0
+#define FLB_KUBE_PROP_STREAM_STDOUT 1
+#define FLB_KUBE_PROP_STREAM_STDERR 2
+#define FLB_KUBE_PROP_STREAM_UNKNOWN 3
+
 int flb_kube_prop_set(struct flb_kube *ctx, struct flb_kube_meta *meta,
                       const char *prop, int prop_len,
                       const char *val_buf, size_t val_len,
