@@ -2,6 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
+ *  Copyright (C) 2019      The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,13 +22,16 @@
 #include <fluent-bit/flb_input.h>
 #include <fluent-bit/flb_network.h>
 
+#ifdef FLB_HAVE_UNIX_SOCKET
 #include <sys/socket.h>
 #include <sys/un.h>
+#endif
 
 #include "fw.h"
 #include "fw_conn.h"
 #include "fw_config.h"
 
+#ifdef FLB_HAVE_UNIX_SOCKET
 static int fw_unix_create(struct flb_in_fw_config *ctx)
 {
     flb_sockfd_t fd = -1;
@@ -62,6 +66,7 @@ static int fw_unix_create(struct flb_in_fw_config *ctx)
     }
     return 0;
 }
+#endif
 
 /*
  * For a server event, the collection event means a new client have arrived, we
@@ -112,6 +117,12 @@ static int in_fw_init(struct flb_input_instance *in,
 
     /* Unix Socket mode */
     if (ctx->unix_path) {
+#ifndef FLB_HAVE_UNIX_SOCKET
+        flb_error("[in_fw] unix address is not supported %s:%s. Aborting",
+            ctx->listen, ctx->tcp_port);
+        fw_config_destroy(ctx);
+        return -1;
+#else
         ret = fw_unix_create(ctx);
         if (ret != 0) {
             flb_error("[in_fw] could not listen on unix://%s",
@@ -120,6 +131,7 @@ static int in_fw_init(struct flb_input_instance *in,
             return -1;
         }
         flb_info("[in_fw] listening on unix://%s", ctx->unix_path);
+#endif
     }
     else {
         /* Create TCP server */
@@ -178,5 +190,5 @@ struct flb_input_plugin in_forward_plugin = {
     .cb_collect   = in_fw_collect,
     .cb_flush_buf = NULL,
     .cb_exit      = in_fw_exit,
-    .flags        = FLB_INPUT_NET | FLB_INPUT_DYN_TAG
+    .flags        = FLB_INPUT_NET
 };
