@@ -33,71 +33,71 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-
 #include "in_head.h"
-
 #define BUF_SIZE_MAX 512
-static int read_lines(struct flb_in_head_config *head_config)
+
+static int read_lines(struct flb_in_head_config *ctx)
 {
     FILE *fp = NULL;
     int i;
     int index = 0;
     int str_len;
     char buf[BUF_SIZE_MAX] = {0};
-
     int  new_len = 0;
     char *tmp;
     char *ret_buf;
 
-    fp = fopen(head_config->filepath, "r");
+    fp = fopen(ctx->filepath, "r");
     if (fp == NULL) {
-        perror("fopen");
+        flb_errno();
         return -1;
     }
 
-    for(i=0; i<head_config->lines; i++){
+    for (i = 0; i<ctx->lines; i++){
         ret_buf = fgets(buf, BUF_SIZE_MAX-1, fp);
         if (ret_buf == NULL) {
             break;
         }
         str_len = strlen(buf);
-        if (head_config->buf_size < str_len + index + 1) {
+        if (ctx->buf_size < str_len + index + 1) {
             /* buffer full. re-allocate new buffer */
-            new_len = head_config->buf_size + str_len + 1;
-            tmp = (char*)flb_malloc(new_len);
+            new_len = ctx->buf_size + str_len + 1;
+            tmp = flb_malloc(new_len);
             if (tmp == NULL) {
                 flb_error("failed to allocate buffer");
                 /* try to output partial data */
                 break;
             }
             /* copy and release old buffer */
-            strcpy(tmp, head_config->buf);
-            flb_free(head_config->buf);
+            strcpy(tmp, ctx->buf);
+            flb_free(ctx->buf);
 
-            head_config->buf_size = new_len;
-            head_config->buf      = tmp;
+            ctx->buf_size = new_len;
+            ctx->buf      = tmp;
         }
-        strncat(&head_config->buf[index], buf, str_len);
-        head_config->buf_len += str_len;
+        strncat(&ctx->buf[index], buf, str_len);
+        ctx->buf_len += str_len;
         index += str_len;
     }
+
     fclose(fp);
     return 0;
 }
 
-static int read_bytes(struct flb_in_head_config *head_config)
+static int read_bytes(struct flb_in_head_config *ctx)
 {
     int fd = -1;
     /* open at every collect callback */
-    fd = open(head_config->filepath, O_RDONLY);
+    fd = open(ctx->filepath, O_RDONLY);
     if (fd < 0) {
-        perror("open");
+        flb_errno();
         return -1;
     }
-    head_config->buf_len = read(fd, head_config->buf, head_config->buf_size);
+    ctx->buf_len = read(fd, ctx->buf, ctx->buf_size);
     close(fd);
-    if (head_config->buf_len < 0) {
-        perror("read");
+
+    if (ctx->buf_len < 0) {
+        flb_errno();
         return -1;
     }
     else {
@@ -106,27 +106,27 @@ static int read_bytes(struct flb_in_head_config *head_config)
 }
 
 static int single_value_per_record(struct flb_input_instance *i_ins,
-                      struct flb_in_head_config *head_config)
+                                   struct flb_in_head_config *ctx)
 {
     int ret = -1;
     int num_map = 1;
     msgpack_packer mp_pck;
     msgpack_sbuffer mp_sbuf;
 
-    head_config->buf[0] = '\0'; /* clear buf */
-    head_config->buf_len =   0;
+    ctx->buf[0] = '\0'; /* clear buf */
+    ctx->buf_len =   0;
 
-    if (head_config->lines > 0) {
-        read_lines(head_config);
+    if (ctx->lines > 0) {
+        read_lines(ctx);
     }
     else {
-        read_bytes(head_config);
+        read_bytes(ctx);
     }
 
     flb_trace("%s read_len=%d buf_size=%d", __FUNCTION__,
-              head_config->buf_len, head_config->buf_size);
+              ctx->buf_len, ctx->buf_size);
 
-    if (head_config->add_path == FLB_TRUE) {
+    if (ctx->add_path == FLB_TRUE) {
         num_map++;
     }
 
@@ -139,19 +139,19 @@ static int single_value_per_record(struct flb_input_instance *i_ins,
     flb_pack_time_now(&mp_pck);
     msgpack_pack_map(&mp_pck, num_map);
 
-    msgpack_pack_str(&mp_pck, head_config->key_len);
-    msgpack_pack_str_body(&mp_pck, head_config->key,
-                          head_config->key_len);
-    msgpack_pack_str(&mp_pck, head_config->buf_len);
+    msgpack_pack_str(&mp_pck, ctx->key_len);
+    msgpack_pack_str_body(&mp_pck, ctx->key,
+                          ctx->key_len);
+    msgpack_pack_str(&mp_pck, ctx->buf_len);
     msgpack_pack_str_body(&mp_pck,
-                          head_config->buf, head_config->buf_len);
+                          ctx->buf, ctx->buf_len);
 
-    if (head_config->add_path == FLB_TRUE) {
+    if (ctx->add_path == FLB_TRUE) {
         msgpack_pack_str(&mp_pck, 4);
         msgpack_pack_str_body(&mp_pck, "path", 4);
-        msgpack_pack_str(&mp_pck, head_config->path_len);
+        msgpack_pack_str(&mp_pck, ctx->path_len);
         msgpack_pack_str_body(&mp_pck,
-                              head_config->filepath, head_config->path_len);
+                              ctx->filepath, ctx->path_len);
     }
 
     ret = 0;
@@ -165,25 +165,25 @@ static int single_value_per_record(struct flb_input_instance *i_ins,
 
 #define KEY_LEN_MAX 32
 static int split_lines_per_record(struct flb_input_instance *i_ins,
-                      struct flb_in_head_config *head_config)
+                      struct flb_in_head_config *ctx)
 {
     FILE *fp = NULL;
     int i;
     size_t str_len;
     size_t key_len;
-    int num_map = head_config->lines;
+    int num_map = ctx->lines;
     char *ret_buf;
     char key_str[KEY_LEN_MAX] = {0};
     msgpack_packer mp_pck;
     msgpack_sbuffer mp_sbuf;
 
-    fp = fopen(head_config->filepath, "r");
+    fp = fopen(ctx->filepath, "r");
     if (fp == NULL) {
-        perror("fopen");
+        flb_errno();
         return -1;
     }
 
-    if (head_config->add_path == FLB_TRUE) {
+    if (ctx->add_path == FLB_TRUE) {
         num_map++;
     }
 
@@ -196,23 +196,23 @@ static int split_lines_per_record(struct flb_input_instance *i_ins,
     flb_pack_time_now(&mp_pck);
     msgpack_pack_map(&mp_pck, num_map);
 
-    if (head_config->add_path == FLB_TRUE) {
+    if (ctx->add_path == FLB_TRUE) {
         msgpack_pack_str(&mp_pck, 4);
         msgpack_pack_str_body(&mp_pck, "path", 4);
-        msgpack_pack_str(&mp_pck, head_config->path_len);
+        msgpack_pack_str(&mp_pck, ctx->path_len);
         msgpack_pack_str_body(&mp_pck,
-                              head_config->filepath, head_config->path_len);
+                              ctx->filepath, ctx->path_len);
     }
 
-    for (i = 0; i < head_config->lines; i++) {
-        ret_buf = fgets(head_config->buf, head_config->buf_size, fp);
+    for (i = 0; i < ctx->lines; i++) {
+        ret_buf = fgets(ctx->buf, ctx->buf_size, fp);
         if (ret_buf == NULL) {
-            head_config->buf[0] = '\0';
+            ctx->buf[0] = '\0';
             str_len = 0;
         }
         else {
-            str_len = strnlen(head_config->buf, head_config->buf_size-1);
-            head_config->buf[str_len-1] = '\0';/* chomp str */
+            str_len = strnlen(ctx->buf, ctx->buf_size-1);
+            ctx->buf[str_len-1] = '\0';/* chomp str */
         }
 
         key_len = snprintf(key_str, KEY_LEN_MAX, "line%d", i);
@@ -224,7 +224,7 @@ static int split_lines_per_record(struct flb_input_instance *i_ins,
         msgpack_pack_str_body(&mp_pck, key_str, key_len);
         msgpack_pack_str(&mp_pck, str_len);
         msgpack_pack_str_body(&mp_pck,
-                              head_config->buf, str_len);
+                              ctx->buf, str_len);
     }
 
     flb_input_chunk_append_raw(i_ins, NULL, 0, mp_sbuf.data, mp_sbuf.size);
@@ -238,21 +238,21 @@ static int split_lines_per_record(struct flb_input_instance *i_ins,
 static int in_head_collect(struct flb_input_instance *i_ins,
                            struct flb_config *config, void *in_context)
 {
-    struct flb_in_head_config *head_config = in_context;
     int ret = -1;
+    struct flb_in_head_config *ctx = in_context;
 
-    if (head_config->lines > 0 && head_config->split_line) {
-        ret = split_lines_per_record(i_ins, head_config);
+    if (ctx->lines > 0 && ctx->split_line) {
+        ret = split_lines_per_record(i_ins, ctx);
     }
     else {
-        ret = single_value_per_record(i_ins, head_config);
+        ret = single_value_per_record(i_ins, ctx);
     }
 
     return ret;
 }
 
 /* read config file and*/
-static int in_head_config_read(struct flb_in_head_config *head_config,
+static int in_head_config_read(struct flb_in_head_config *ctx,
                                struct flb_input_instance *in)
 {
     const char *filepath = NULL;
@@ -263,146 +263,169 @@ static int in_head_config_read(struct flb_in_head_config *head_config,
     if (!filepath) {
         return -1;
     }
-    head_config->filepath = filepath;
+    ctx->filepath = filepath;
 
     pval = flb_input_get_property("key", in);
     if (pval) {
-        head_config->key      = pval;
-        head_config->key_len  = strlen(pval);
+        ctx->key      = pval;
+        ctx->key_len  = strlen(pval);
     }
     else {
-        head_config->key      = "head";
-        head_config->key_len  = 4;
+        ctx->key      = "head";
+        ctx->key_len  = 4;
     }
 
     /* buffer size setting */
     pval = flb_input_get_property("buf_size", in);
     if (pval != NULL && atoi(pval) > 0) {
-        head_config->buf_size = atoi(pval);
+        ctx->buf_size = atoi(pval);
     }
     else {
-        head_config->buf_size = DEFAULT_BUF_SIZE;
+        ctx->buf_size = DEFAULT_BUF_SIZE;
     }
 
     /* interval settings */
     pval = flb_input_get_property("interval_sec", in);
     if (pval != NULL && atoi(pval) >= 0) {
-        head_config->interval_sec = atoi(pval);
+        ctx->interval_sec = atoi(pval);
     }
     else {
-        head_config->interval_sec = DEFAULT_INTERVAL_SEC;
+        ctx->interval_sec = DEFAULT_INTERVAL_SEC;
     }
 
     pval = flb_input_get_property("interval_nsec", in);
     if (pval != NULL && atoi(pval) >= 0) {
-        head_config->interval_nsec = atoi(pval);
+        ctx->interval_nsec = atoi(pval);
     }
     else {
-        head_config->interval_nsec = DEFAULT_INTERVAL_NSEC;
+        ctx->interval_nsec = DEFAULT_INTERVAL_NSEC;
     }
 
     pval = flb_input_get_property("split_line", in);
     if (pval != NULL && flb_utils_bool(pval)) {
-        head_config->split_line = FLB_TRUE;
-        head_config->lines      = 10;
+        ctx->split_line = FLB_TRUE;
+        ctx->lines      = 10;
     }
     else {
-        head_config->split_line = FLB_FALSE;
+        ctx->split_line = FLB_FALSE;
     }
 
     pval = flb_input_get_property("lines", in);
     if (pval != NULL && atoi(pval) >= 0) {
-        head_config->lines = atoi(pval);
+        ctx->lines = atoi(pval);
     }
     else {
-        head_config->lines = 0; /* read bytes mode */
+        ctx->lines = 0; /* read bytes mode */
     }
 
-    if (head_config->interval_sec <= 0 && head_config->interval_nsec <= 0) {
+    if (ctx->interval_sec <= 0 && ctx->interval_nsec <= 0) {
         /* Illegal settings. Override them. */
-        head_config->interval_sec = DEFAULT_INTERVAL_SEC;
-        head_config->interval_nsec = DEFAULT_INTERVAL_NSEC;
+        ctx->interval_sec = DEFAULT_INTERVAL_SEC;
+        ctx->interval_nsec = DEFAULT_INTERVAL_NSEC;
     }
 
     pval = flb_input_get_property("add_path", in);
     if (pval) {
         if (strcasecmp(pval, "true") == 0 || strcasecmp(pval, "on") == 0) {
-            head_config->add_path = FLB_TRUE;
-            head_config->path_len = strlen(head_config->filepath);
+            ctx->add_path = FLB_TRUE;
+            ctx->path_len = strlen(ctx->filepath);
         }
     }
 
     flb_debug("[in_head] buf_size=%d path=%s",
-              head_config->buf_size, head_config->filepath);
+              ctx->buf_size, ctx->filepath);
     flb_debug("[in_head] interval_sec=%d interval_nsec=%d",
-              head_config->interval_sec, head_config->interval_nsec);
+              ctx->interval_sec, ctx->interval_nsec);
 
     return 0;
 }
 
-static void delete_head_config(struct flb_in_head_config *head_config)
+static void delete_head_config(struct flb_in_head_config *ctx)
 {
-    if (head_config) {
-        /* release buffer */
-        if (head_config->buf != NULL) {
-            flb_free(head_config->buf);
-        }
-        flb_free(head_config);
+    if (!ctx) {
+        return;
     }
+
+    /* release buffer */
+    if (ctx->buf) {
+        flb_free(ctx->buf);
+    }
+    flb_free(ctx);
 }
 
 /* Initialize plugin */
 static int in_head_init(struct flb_input_instance *in,
                         struct flb_config *config, void *data)
 {
-    struct flb_in_head_config *head_config = NULL;
     int ret = -1;
+    struct flb_in_head_config *ctx;
 
     /* Allocate space for the configuration */
-    head_config = flb_malloc(sizeof(struct flb_in_head_config));
-    if (head_config == NULL) {
+    ctx = flb_calloc(1, sizeof(struct flb_in_head_config));
+    if (!ctx) {
         return -1;
     }
-    head_config->buf = NULL;
-    head_config->buf_len = 0;
-    head_config->add_path = FLB_FALSE;
-    head_config->lines = 0;
+
+    ctx->buf = NULL;
+    ctx->buf_len = 0;
+    ctx->add_path = FLB_FALSE;
+    ctx->lines = 0;
+    ctx->ins= in;
 
     /* Initialize head config */
-    ret = in_head_config_read(head_config, in);
+    ret = in_head_config_read(ctx, in);
     if (ret < 0) {
         goto init_error;
     }
 
-    head_config->buf = flb_malloc(head_config->buf_size);
-    if (head_config->buf == NULL) {
-        flb_error("could not allocate head buffer");
+    ctx->buf = flb_malloc(ctx->buf_size);
+    if (!ctx->buf) {
+        flb_errno();
         goto init_error;
     }
 
-    flb_trace("%s read_len=%d buf_size=%d", __FUNCTION__,
-              head_config->buf_len, sizeof(head_config->buf));
+    flb_trace("%s read_len=%d buf_size=%d", __FUNCTION__, ctx->buf_len,
+              ctx->buf_size);
 
-    flb_input_set_context(in, head_config);
+    flb_input_set_context(in, ctx);
 
     ret = flb_input_set_collector_time(in,
                                        in_head_collect,
-                                       head_config->interval_sec,
-                                       head_config->interval_nsec, config);
+                                       ctx->interval_sec,
+                                       ctx->interval_nsec, config);
     if (ret < 0) {
         flb_error("could not set collector for head input plugin");
         goto init_error;
     }
 
+    ctx->coll_fd = ret;
     return 0;
 
   init_error:
-    delete_head_config(head_config);
+    delete_head_config(ctx);
 
     return -1;
 }
 
-int in_head_exit(void *data, struct flb_config *config)
+static void in_head_pause(void *data, struct flb_config *config)
+{
+    struct flb_in_head_config *ctx = data;
+    (void) config;
+
+    /* Pause collector */
+    flb_input_collector_pause(ctx->coll_fd, ctx->ins);
+}
+
+static void in_head_resume(void *data, struct flb_config *config)
+{
+    struct flb_in_head_config *ctx = data;
+    (void) config;
+
+    /* Resume collector */
+    flb_input_collector_resume(ctx->coll_fd, ctx->ins);
+}
+
+static int in_head_exit(void *data, struct flb_config *config)
 {
     (void) *config;
     struct flb_in_head_config *head_config = data;
@@ -419,5 +442,7 @@ struct flb_input_plugin in_head_plugin = {
     .cb_pre_run   = NULL,
     .cb_collect   = in_head_collect,
     .cb_flush_buf = NULL,
+    .cb_pause     = in_head_pause,
+    .cb_resume    = in_head_resume,
     .cb_exit      = in_head_exit
 };
