@@ -29,11 +29,17 @@
 static char *human_readable_size(long size)
 {
     long u = 1024, i, len = 128;
-    char *buf = flb_malloc(len);
+    char *buf;
     static const char *__units[] = { "b", "K", "M", "G",
-        "T", "P", "E", "Z", "Y", NULL
+                                     "T", "P", "E", "Z", "Y", NULL
     };
 
+    buf = flb_malloc(len);
+    if (!buf) {
+        flb_errno();
+        return NULL;
+    }
+
     for (i = 0; __units[i] != NULL; i++) {
         if ((size / u) == 0) {
             break;
@@ -163,6 +169,11 @@ struct proc_task *proc_stat(pid_t pid, int page_size)
     /* Internal conversion */
     t->proc_rss    = (t->rss * page_size);
     t->proc_rss_hr = human_readable_size(t->proc_rss);
+    if ( t->proc_rss_hr == NULL ) {
+        flb_free(buf);
+        flb_free(t);
+        return NULL;
+    }
 
     flb_free(buf);
     return t;
