@@ -2,6 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
+ *  Copyright (C) 2019      The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -27,11 +28,14 @@
 struct flb_in_mqtt_config *mqtt_config_init(struct flb_input_instance *i_ins)
 {
     char tmp[16];
-    char *listen;
+    const char *listen;
     struct flb_in_mqtt_config *config;
 
-    config = flb_malloc(sizeof(struct flb_in_mqtt_config));
-    memset(config, '\0', sizeof(struct flb_in_mqtt_config));
+    config = flb_calloc(1, sizeof(struct flb_in_mqtt_config));
+    if (!config) {
+        flb_errno();
+        return NULL;
+    }
 
     /* Listen interface (if not set, defaults to 0.0.0.0) */
     if (!i_ins->host.listen) {
@@ -59,6 +63,7 @@ struct flb_in_mqtt_config *mqtt_config_init(struct flb_input_instance *i_ins)
     flb_debug("[in_mqtt] Listen='%s' TCP_Port=%s",
               config->listen, config->tcp_port);
 
+    mk_list_init(&config->conns);
     return config;
 }
 
