@@ -269,8 +269,8 @@ static int mqtt_handle_publish(struct mqtt_conn *conn)
             mqtt_packet_header(MQTT_PUBREC, 2 , (char *) &buf);
         }
         /* Set the identifier that we are replying to */
-        buf[2] = (packet_id & 0xf0) >> 4;
-        buf[3] = (packet_id & 0xf);
+        buf[2] = (packet_id >> 8) & 0xff;
+        buf[3] = (packet_id & 0xff);
         write(conn->event.fd, buf, 4);
     }
 
@@ -291,7 +291,7 @@ static int mqtt_handle_ping(struct mqtt_conn *conn)
     int ret;
     char buf[2] = {0, 0};
 
-    mqtt_packet_header(MQTT_PINGRESP, 2 , (char *) &buf);
+    mqtt_packet_header(MQTT_PINGRESP, 0 , (char *) &buf);
 
     /* write PINGRESP message */
     ret = write(conn->event.fd, buf, 2);
