@@ -2,6 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
+ *  Copyright (C) 2019      The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -47,7 +48,7 @@ struct netif_entry {
 };
 
 struct flb_in_netif_config {
-    char *interface;
+    const char *interface;
     int  interface_len;
 
     int  verbose;
