@@ -18,6 +18,8 @@
  *  limitations under the License.
  */
 
+#include <fluent-bit.h>
+
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_log.h>
 #include <fluent-bit/flb_input.h>
@@ -54,7 +56,9 @@ struct flb_tail_config *flb_tail_config_create(struct flb_input_instance *i_ins,
     ctx->i_ins = i_ins;
     ctx->ignore_older = 0;
     ctx->skip_long_lines = FLB_FALSE;
+#ifdef FLB_HAVE_SQLDB
     ctx->db_sync = -1;
+#endif
 
     /* Create the channel manager */
     ret = flb_pipe_create(ctx->ch_manager);
@@ -63,6 +67,8 @@ struct flb_tail_config *flb_tail_config_create(struct flb_input_instance *i_ins,
         flb_free(ctx);
         return NULL;
     }
+    ctx->ch_reads = 0;
+    ctx->ch_writes = 0;
 
     /* Create the pending channel */
     ret = flb_pipe_create(ctx->ch_pending);
@@ -260,7 +266,9 @@ struct flb_tail_config *flb_tail_config_create(struct flb_input_instance *i_ins,
     mk_list_init(&ctx->files_static);
     mk_list_init(&ctx->files_event);
     mk_list_init(&ctx->files_rotated);
+#ifdef FLB_HAVE_SQLDB
     ctx->db = NULL;
+#endif
 
 #ifdef FLB_HAVE_REGEX
     tmp = flb_input_get_property("tag_regex", i_ins);
@@ -284,6 +292,7 @@ struct flb_tail_config *flb_tail_config_create(struct flb_input_instance *i_ins,
         ctx->dynamic_tag = FLB_TRUE;
     }
 
+#ifdef FLB_HAVE_SQLDB
     /* Database options (needs to be set before the context) */
     tmp = flb_input_get_property("db.sync", i_ins);
     if (tmp) {
@@ -312,6 +321,7 @@ struct flb_tail_config *flb_tail_config_create(struct flb_input_instance *i_ins,
             flb_error("[in_tail] could not open/create database");
         }
     }
+#endif
 
 #ifdef FLB_HAVE_METRICS
     flb_metrics_add(FLB_TAIL_METRIC_F_OPENED,
@@ -344,9 +354,11 @@ int flb_tail_config_destroy(struct flb_tail_config *config)
     }
 #endif
 
+#ifdef FLB_HAVE_SQLDB
     if (config->db != NULL) {
         flb_tail_db_close(config->db);
     }
+#endif
 
     if (config->key != NULL) {
         flb_free(config->key);
