@@ -45,6 +45,8 @@ struct flb_tail_config {
     int ch_manager[2];         /* pipe: channel manager    */
     int ch_pending[2];         /* pipe: pending events     */
 #endif
+    int ch_reads;              /* count number if signal reads */
+    int ch_writes;             /* count number of signal writes */
 
     /* Buffer Config */
     size_t buf_chunk_size;     /* allocation chunks        */
@@ -82,8 +84,10 @@ struct flb_tail_config {
     int   exit_on_eof;         /* exit fluent-bit on EOF, test */
 
     /* Database */
+#ifdef FLB_HAVE_SQLDB
     struct flb_sqldb *db;
     int db_sync;
+#endif
 
     /* Parser / Format */
     struct flb_parser *parser;
