@@ -68,6 +68,14 @@ struct flb_sqldb *flb_tail_db_open(const char *path,
         }
     }
 
+
+    ret = flb_sqldb_query(db, SQL_PRAGMA_JOURNAL_MODE, NULL, NULL);
+    if (ret != FLB_OK) {
+        flb_error("[in_tail:db] could not set pragma 'journal_mode'");
+        flb_sqldb_close(db);
+        return NULL;
+    }
+
     return db;
 }
 
@@ -100,7 +108,7 @@ int flb_tail_db_file_set(struct flb_tail_file *file,
     /* Check if the file exists */
     snprintf(query, sizeof(query) - 1,
              SQL_GET_FILE,
-             file->name, file->inode);
+             file->name, (uint64_t) file->inode);
 
     memset(&qs, '\0', sizeof(qs));
     ret = flb_sqldb_query(ctx->db,
@@ -143,7 +151,6 @@ int flb_tail_db_file_offset(struct flb_tail_file *file,
     if (ret == FLB_ERROR) {
         return -1;
     }
-
     return 0;
 }
 
@@ -170,3 +177,22 @@ int flb_tail_db_file_rotate(const char *new_name,
 
     return 0;
 }
+
+/* Delete file entry from the database */
+int flb_tail_db_file_delete(struct flb_tail_file *file,
+                            struct flb_tail_config *ctx)
+{
+    int ret;
+    char query[PATH_MAX];
+
+    /* Check if the file exists */
+    snprintf(query, sizeof(query) - 1, SQL_DELETE_FILE, file->db_id);
+    ret = flb_sqldb_query(ctx->db, query, NULL, NULL);
+    if (ret != FLB_OK) {
+        flb_error("[in_tail:db] error deleting entry from database: %s", file->name);
+        return -1;
+    }
+
+    flb_debug("[in_tail:db] file deleted from database: %s", file->name);
+    return 0;
+}
