@@ -28,12 +28,30 @@ static inline int tail_signal_manager(struct flb_tail_config *ctx)
     int n;
     uint64_t val = 0xc001;
 
+    /*
+     * The number of signal reads might be less than the written signals, this
+     * means that some event is still pending in the queue. On that case we
+     * don't need to signal it again.
+     */
+    if (ctx->ch_reads < ctx->ch_writes) {
+        return 1;
+    }
+
+    /* Reset counters: prevent an overflow, unlikely..but let's keep safe */
+    if (ctx->ch_reads == ctx->ch_writes) {
+        ctx->ch_reads = 0;
+        ctx->ch_writes = 0;
+    }
+
     /* Insert a dummy event into the channel manager */
     n = flb_pipe_w(ctx->ch_manager[1], &val, sizeof(val));
     if (n == -1) {
         flb_errno();
         return -1;
     }
+    else {
+        ctx->ch_writes++;
+    }
 
     return n;
 }
