@@ -57,12 +57,18 @@ static inline int process_pack(struct tcp_conn *conn,
         if (entry.type == MSGPACK_OBJECT_MAP) {
             msgpack_pack_object(&mp_pck, entry);
         }
-        else {
+        else if (entry.type == MSGPACK_OBJECT_ARRAY) {
             msgpack_pack_map(&mp_pck, 1);
             msgpack_pack_str(&mp_pck, 3);
             msgpack_pack_str_body(&mp_pck, "msg", 3);
             msgpack_pack_object(&mp_pck, entry);
         }
+        else {
+            flb_debug("[in_tcp] record is not a JSON map or array");
+            msgpack_unpacked_destroy(&result);
+            msgpack_sbuffer_destroy(&mp_sbuf);
+            return -1;
+        }
     }
 
     msgpack_unpacked_destroy(&result);
@@ -73,13 +79,94 @@ static inline int process_pack(struct tcp_conn *conn,
     return 0;
 }
 
+/* Process a JSON payload, return the number of processed bytes */
+static ssize_t parse_payload_json(struct tcp_conn *conn)
+{
+    int ret;
+    int out_size;
+    char *pack;
+
+    ret = flb_pack_json_state(conn->buf_data, conn->buf_len,
+                              &pack, &out_size, &conn->pack_state);
+    if (ret == FLB_ERR_JSON_PART) {
+        flb_debug("[in_tcp] JSON incomplete, waiting for more data...");
+        return 0;
+    }
+    else if (ret == FLB_ERR_JSON_INVAL) {
+        flb_warn("[in_tcp] invalid JSON message, skipping");
+        conn->buf_len = 0;
+        conn->pack_state.multiple = FLB_TRUE;
+        return -1;
+    }
+    else if (ret == -1) {
+        return -1;
+    }
+
+    /* Process the packaged JSON and return the last byte used */
+    process_pack(conn, pack, out_size);
+    flb_free(pack);
+
+    return conn->pack_state.last_byte;
+}
+
+/*
+ * Process a raw text payload, uses the delimited character to split records,
+ * return the number of processed bytes
+ */
+static ssize_t parse_payload_none(struct tcp_conn *conn)
+{
+    int len;
+    int sep_len;
+    size_t consumed = 0;
+    char *buf;
+    char *s;
+    char *separator;
+    msgpack_packer mp_pck;
+    msgpack_sbuffer mp_sbuf;
+
+    separator = conn->ctx->separator;
+    sep_len = flb_sds_len(conn->ctx->separator);
+
+    /* Initialize local msgpack buffer */
+    msgpack_sbuffer_init(&mp_sbuf);
+    msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
+
+    buf = conn->buf_data;
+
+    while ((s = strstr(buf, separator))) {
+        len = (s - buf);
+        if (len == 0) {
+            break;
+        }
+        else if (len > 0) {
+            msgpack_pack_array(&mp_pck, 2);
+            flb_pack_time_now(&mp_pck);
+            msgpack_pack_map(&mp_pck, 1);
+            msgpack_pack_str(&mp_pck, 3);
+            msgpack_pack_str_body(&mp_pck, "log", 3);
+            msgpack_pack_str(&mp_pck, len);
+            msgpack_pack_str_body(&mp_pck, buf, len);
+            consumed += len + 1;
+            buf += len + sep_len;
+        }
+        else {
+            break;
+        }
+    }
+
+    flb_input_chunk_append_raw(conn->in, NULL, 0, mp_sbuf.data, mp_sbuf.size);
+    msgpack_sbuffer_destroy(&mp_sbuf);
+
+    return consumed;
+}
+
 /* Callback invoked every time an event is triggered for a connection */
 int tcp_conn_event(void *data)
 {
-    int ret;
     int bytes;
     int available;
     int size;
+    ssize_t ret_payload = -1;
     char *tmp;
     struct mk_event *event;
     struct tcp_conn *conn = data;
@@ -87,7 +174,7 @@ int tcp_conn_event(void *data)
 
     event = &conn->event;
     if (event->mask & MK_EVENT_READ) {
-        available = (conn->buf_size - conn->buf_len);
+        available = (conn->buf_size - conn->buf_len) - 1;
         if (available < 1) {
             if (conn->buf_size + ctx->chunk_size > ctx->buffer_size) {
                 flb_trace("[in_tcp] fd=%i incoming data exceed limit (%i KB)",
@@ -107,7 +194,7 @@ int tcp_conn_event(void *data)
 
             conn->buf_data = tmp;
             conn->buf_size = size;
-            available = (conn->buf_size - conn->buf_len);
+            available = (conn->buf_size - conn->buf_len) - 1;
         }
 
         /* Read data */
@@ -131,41 +218,45 @@ int tcp_conn_event(void *data)
                       conn->buf_data[0]);
             consume_bytes(conn->buf_data, 1, conn->buf_len);
             conn->buf_len--;
+            conn->buf_data[conn->buf_len] = '\0';
         }
 
         /* JSON Format handler */
-        char *pack;
-        int out_size;
-        ret = flb_pack_json_state(conn->buf_data, conn->buf_len,
-                                  &pack, &out_size, &conn->pack_state);
-        if (ret == FLB_ERR_JSON_PART) {
-            flb_debug("[in_tcp] JSON incomplete, waiting for more data...");
-            return 0;
+        if (ctx->format == FLB_TCP_FMT_JSON) {
+            ret_payload = parse_payload_json(conn);
+            if (ret_payload == 0) {
+                /* Incomplete JSON message, we need more data */
+                return -1;
+            }
+            else if (ret_payload == -1) {
+                flb_pack_state_reset(&conn->pack_state);
+                flb_pack_state_init(&conn->pack_state);
+                conn->pack_state.multiple = FLB_TRUE;
+                return -1;
+            }
         }
-        else if (ret == FLB_ERR_JSON_INVAL) {
-            flb_warn("[in_tcp] invalid JSON message, skipping");
-            flb_pack_state_reset(&conn->pack_state);
-            flb_pack_state_init(&conn->pack_state);
-            conn->buf_len = 0;
-            conn->pack_state.multiple = FLB_TRUE;
-            return -1;
+        else if (ctx->format == FLB_TCP_FMT_NONE) {
+            ret_payload = parse_payload_none(conn);
+            if (ret_payload == 0) {
+                return -1;
+            }
+            else if (ret_payload == -1) {
+                conn->buf_len = 0;
+                return -1;
+            }
         }
 
-        /*
-         * Given the Tokens used for the packaged message, append
-         * the records and then adjust buffer.
-         */
-        process_pack(conn, pack, out_size);
-
-        consume_bytes(conn->buf_data, conn->pack_state.last_byte, conn->buf_len);
-        conn->buf_len -= conn->pack_state.last_byte;
+        consume_bytes(conn->buf_data, ret_payload, conn->buf_len);
+        conn->buf_len -= ret_payload;
         conn->buf_data[conn->buf_len] = '\0';
 
-        flb_pack_state_reset(&conn->pack_state);
-        flb_pack_state_init(&conn->pack_state);
-        conn->pack_state.multiple = FLB_TRUE;
+        if (ctx->format == FLB_TCP_FMT_JSON) {
+            jsmn_init(&conn->pack_state.parser);
+            conn->pack_state.tokens_count = 0;
+            conn->pack_state.last_byte = 0;
+            conn->pack_state.buf_len = 0;
+        }
 
-        flb_free(pack);
         return bytes;
     }
 
@@ -174,6 +265,7 @@ int tcp_conn_event(void *data)
         tcp_conn_del(conn);
         return -1;
     }
+
     return 0;
 }
 
@@ -216,8 +308,10 @@ struct tcp_conn *tcp_conn_add(int fd, struct flb_in_tcp_config *ctx)
     conn->in       = ctx->in;
 
     /* Initialize JSON parser */
-    flb_pack_state_init(&conn->pack_state);
-    conn->pack_state.multiple = FLB_TRUE;
+    if (ctx->format == FLB_TCP_FMT_JSON) {
+        flb_pack_state_init(&conn->pack_state);
+        conn->pack_state.multiple = FLB_TRUE;
+    }
 
     /* Register instance into the event loop */
     ret = mk_event_add(ctx->evl, fd, FLB_ENGINE_EV_CUSTOM, MK_EVENT_READ, conn);
@@ -240,8 +334,9 @@ int tcp_conn_del(struct tcp_conn *conn)
 
     ctx = conn->ctx;
 
-    flb_pack_state_reset(&conn->pack_state);
-
+    if (ctx->format == FLB_TCP_FMT_JSON) {
+        flb_pack_state_reset(&conn->pack_state);
+    }
     /* Unregister the file descriptior from the event-loop */
     mk_event_del(ctx->evl, &conn->event);
 
