@@ -135,8 +135,8 @@ static char *elasticsearch_format(const void *data, size_t bytes,
     int ret;
     int len;
     int map_size;
-    int index_len;
-    size_t s;
+    int index_len = 0;
+    size_t s = 0;
     size_t off = 0;
     char *p;
     char *buf;
@@ -215,8 +215,8 @@ static char *elasticsearch_format(const void *data, size_t bytes,
     if (ctx->logstash_format == FLB_FALSE && ctx->generate_id == FLB_FALSE) {
         flb_time_get(&tms);
         gmtime_r(&tms.tm.tv_sec, &tm);
-        s = strftime(index_formatted, sizeof(index_formatted) - 1,
-                     ctx->index, &tm);
+        strftime(index_formatted, sizeof(index_formatted) - 1,
+                 ctx->index, &tm);
         es_index = index_formatted;
 
         index_len = snprintf(j_index,
@@ -419,9 +419,9 @@ static char *elasticsearch_format(const void *data, size_t bytes,
     return buf;
 }
 
-int cb_es_init(struct flb_output_instance *ins,
-               struct flb_config *config,
-               void *data)
+static int cb_es_init(struct flb_output_instance *ins,
+                      struct flb_config *config,
+                      void *data)
 {
     struct flb_elasticsearch *ctx;
 
