@@ -46,9 +46,12 @@ struct flb_forward_config {
     /* config */
     flb_sds_t shared_key;     /* shared key                   */
     flb_sds_t self_hostname;  /* hotname used in certificate  */
+    int empty_shared_key;     /* use an empty string as shared key */
     int require_ack_response; /* Require acknowledge for "chunk" */
     int send_options;         /* send options in messages */
 
+    const char *username;
+    const char *password;
 
     /* mbedTLS specifics */
     unsigned char shared_key_salt[16];
@@ -72,4 +75,11 @@ struct flb_forward {
     struct mk_list configs;
 };
 
+struct flb_forward_ping {
+    const char *nonce;
+    int nonce_len;
+    const char *auth;
+    int auth_len;
+    int keepalive;
+};
 #endif
