@@ -27,6 +27,7 @@
 #include <fluent-bit/flb_utils.h>
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_sds.h>
+#include <fluent-bit/flb_gzip.h>
 #include <msgpack.h>
 
 #include <stdio.h>
@@ -60,11 +61,13 @@ static int http_post(struct flb_out_http *ctx,
 {
     int ret;
     int out_ret = FLB_OK;
+    int compressed = FLB_FALSE;
     size_t b_sent;
+    void *payload_buf = NULL;
+    size_t payload_size = 0;
     struct flb_upstream *u;
     struct flb_upstream_conn *u_conn;
     struct flb_http_client *c;
-
     struct mk_list *tmp;
     struct mk_list *head;
     struct out_http_header *header;
@@ -78,9 +81,25 @@ static int http_post(struct flb_out_http *ctx,
         return FLB_RETRY;
     }
 
+    /* Map payload */
+    payload_buf = (void *) body;
+    payload_size = body_len;
+
+    /* Should we compress the payload ? */
+    if (ctx->compress_gzip == FLB_TRUE) {
+        ret = flb_gzip_compress((void *) body, body_len,
+                                &payload_buf, &payload_size);
+        if (ret == -1) {
+            flb_error("[out_http] cannot gzip payload, disabling compression");
+        }
+        else {
+            compressed = FLB_TRUE;
+        }
+    }
+
     /* Create HTTP client context */
     c = flb_http_client(u_conn, FLB_HTTP_POST, ctx->uri,
-                        body, body_len,
+                        payload_buf, payload_size,
                         ctx->host, ctx->port,
                         ctx->proxy, 0);
 
@@ -110,12 +129,19 @@ static int http_post(struct flb_out_http *ctx,
                         tag, tag_len);
     }
 
+    /* Content Encoding: gzip */
+    if (compressed == FLB_TRUE) {
+        flb_http_set_content_encoding_gzip(c);
+    }
+
+    /* Basic Auth headers */
     if (ctx->http_user && ctx->http_passwd) {
         flb_http_basic_auth(c, ctx->http_user, ctx->http_passwd);
     }
 
     flb_http_add_header(c, "User-Agent", 10, "Fluent-Bit", 10);
 
+
     mk_list_foreach_safe(head, tmp, &ctx->headers) {
         header = mk_list_entry(head, struct out_http_header, _head);
         flb_http_add_header(c,
@@ -162,9 +188,18 @@ static int http_post(struct flb_out_http *ctx,
         out_ret = FLB_RETRY;
     }
 
+    /*
+     * If the payload buffer is different than incoming records in body, means
+     * we generated a different payload and must be freed.
+     */
+    if (payload_buf != body) {
+        flb_free(payload_buf);
+    }
+
+    /* Destroy HTTP client context */
     flb_http_client_destroy(c);
 
-    /* Release the connection */
+    /* Release the TCP connection */
     flb_upstream_conn_release(u_conn);
 
     return out_ret;
@@ -175,10 +210,9 @@ static int http_gelf(struct flb_out_http *ctx,
                      const char *tag, int tag_len)
 {
     flb_sds_t s;
-    flb_sds_t tmp;
+    flb_sds_t tmp = NULL;
     msgpack_unpacked result;
     size_t off = 0;
-    size_t prev_off = 0;
     size_t size = 0;
     msgpack_object root;
     msgpack_object map;
@@ -186,13 +220,18 @@ static int http_gelf(struct flb_out_http *ctx,
     struct flb_time tm;
     int ret;
 
-    msgpack_unpacked_init(&result);
+    size = bytes * 1.5;
 
+    /* Allocate buffer for our new payload */
+    s = flb_sds_create_size(size);
+    if (!s) {
+        return FLB_RETRY;
+    }
+
+    msgpack_unpacked_init(&result);
     while (msgpack_unpack_next(&result, data, bytes, &off) ==
            MSGPACK_UNPACK_SUCCESS) {
 
-        size = off - prev_off;
-        prev_off = off;
         if (result.data.type != MSGPACK_OBJECT_ARRAY) {
             continue;
         }
@@ -205,33 +244,30 @@ static int http_gelf(struct flb_out_http *ctx,
         flb_time_pop_from_msgpack(&tm, &result, &obj);
         map = root.via.array.ptr[1];
 
-        size = (size * 1.4);
-        s = flb_sds_create_size(size);
-        if (s == NULL) {
+        tmp = flb_msgpack_to_gelf(&s, &map, &tm, &(ctx->gelf_fields));
+        if (!tmp) {
+            flb_error("[out_http] error encoding to GELF");
+            flb_sds_destroy(s);
+            msgpack_unpacked_destroy(&result);
+            return FLB_ERROR;
+        }
+
+        /* Append new line */
+        tmp = flb_sds_cat(s, "\n", 1);
+        if (!tmp) {
+            flb_error("[out_http] error concatenating records");
+            flb_sds_destroy(s);
             msgpack_unpacked_destroy(&result);
             return FLB_RETRY;
         }
-
-        tmp = flb_msgpack_to_gelf(&s, &map, &tm, &(ctx->gelf_fields));
-        if (tmp != NULL) {
-            s = tmp;
-            ret = http_post(ctx, s, flb_sds_len(s), tag, tag_len);
-            if (ret != FLB_OK) {
-                msgpack_unpacked_destroy(&result);
-                flb_sds_destroy(s);
-                return ret;
-            }
-        }
-        else {
-            flb_error("[out_http] error encoding to GELF");
-        }
-
-        flb_sds_destroy(s);
+        s = tmp;
     }
 
+    ret = http_post(ctx, s, flb_sds_len(s), tag, tag_len);
+    flb_sds_destroy(s);
     msgpack_unpacked_destroy(&result);
 
-    return FLB_OK;
+    return ret;
 }
 
 static void cb_http_flush(const void *data, size_t bytes,
