@@ -209,7 +209,7 @@ static char *influxdb_format(const char *tag, int tag_len,
                 val = tmp;
                 val_len = snprintf(tmp, sizeof(tmp) - 1, "%" PRId64, v->via.i64);
             }
-            else if (v->type == MSGPACK_OBJECT_FLOAT) {
+            else if (v->type == MSGPACK_OBJECT_FLOAT || v->type == MSGPACK_OBJECT_FLOAT32) {
                 val = tmp;
                 val_len = snprintf(tmp, sizeof(tmp) - 1, "%f", v->via.f64);
             }
@@ -326,22 +326,16 @@ error:
     return NULL;
 }
 
-int cb_influxdb_init(struct flb_output_instance *ins, struct flb_config *config,
-                     void *data)
+static int cb_influxdb_init(struct flb_output_instance *ins, struct flb_config *config,
+                            void *data)
 {
     int io_flags = 0;
     const char *tmp;
     struct flb_upstream *upstream;
     struct flb_influxdb_config *ctx;
 
-    /* Get network configuration */
-    if (!ins->host.name) {
-        ins->host.name = flb_strdup(FLB_INFLUXDB_HOST);
-    }
-
-    if (ins->host.port == 0) {
-        ins->host.port = FLB_INFLUXDB_PORT;
-    }
+    /* Set default network configuration */
+    flb_output_net_default(FLB_INFLUXDB_HOST, FLB_INFLUXDB_PORT, ins);
 
     /* Allocate plugin context */
     ctx = flb_calloc(1, sizeof(struct flb_influxdb_config));
@@ -440,11 +434,11 @@ int cb_influxdb_init(struct flb_output_instance *ins, struct flb_config *config,
     return 0;
 }
 
-void cb_influxdb_flush(const void *data, size_t bytes,
-                       const char *tag, int tag_len,
-                       struct flb_input_instance *i_ins,
-                       void *out_context,
-                       struct flb_config *config)
+static void cb_influxdb_flush(const void *data, size_t bytes,
+                              const char *tag, int tag_len,
+                              struct flb_input_instance *i_ins,
+                              void *out_context,
+                              struct flb_config *config)
 {
     int ret;
     int bytes_out;
@@ -504,7 +498,7 @@ void cb_influxdb_flush(const void *data, size_t bytes,
     FLB_OUTPUT_RETURN(FLB_OK);
 }
 
-int cb_influxdb_exit(void *data, struct flb_config *config)
+static int cb_influxdb_exit(void *data, struct flb_config *config)
 {
     struct flb_influxdb_config *ctx = data;
 
@@ -554,7 +548,7 @@ int is_tagged_key(struct flb_influxdb_config *ctx, const char *key, int kl, int
 
     if (ctx->tag_keys) {
         mk_list_foreach(head, ctx->tag_keys) {
-            entry = mk_list_entry(head, struct flb_split_entry ,_head);
+            entry = mk_list_entry(head, struct flb_split_entry, _head);
             if (kl == entry->len && strncmp(key, entry->value, kl) == 0) {
                 return FLB_TRUE;
             }
