@@ -21,6 +21,7 @@
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_output.h>
 #include <fluent-bit/flb_mem.h>
+#include <fluent-bit/flb_kv.h>
 #include <fluent-bit/flb_utils.h>
 
 #include "kafka_config.h"
@@ -37,7 +38,7 @@ struct flb_kafka *flb_kafka_conf_create(struct flb_output_instance *ins,
     struct mk_list *topics;
     struct flb_split_entry *entry;
     struct flb_kafka *ctx;
-    struct flb_config_prop *prop;
+    struct flb_kv *kv;
 
     /* Configuration context */
     ctx = flb_calloc(1, sizeof(struct flb_kafka));
@@ -84,15 +85,15 @@ struct flb_kafka *flb_kafka_conf_create(struct flb_output_instance *ins,
 
     /* Iterate custom rdkafka properties */
     mk_list_foreach(head, &ins->properties) {
-        prop = mk_list_entry(head, struct flb_config_prop, _head);
-        if (strncasecmp(prop->key, "rdkafka.", 8) == 0 &&
-            strlen(prop->key) > 8) {
+        kv = mk_list_entry(head, struct flb_kv, _head);
+        if (strncasecmp(kv->key, "rdkafka.", 8) == 0 &&
+            flb_sds_len(kv->key) > 8) {
 
-            ret = rd_kafka_conf_set(ctx->conf, prop->key + 8, prop->val,
+            ret = rd_kafka_conf_set(ctx->conf, kv->key + 8, kv->val,
                                     errstr, sizeof(errstr));
             if (ret != RD_KAFKA_CONF_OK) {
                 flb_error("[out_kafka] cannot configure '%s' property",
-                          prop->key + 8);
+                          kv->key + 8);
             }
         }
     }
@@ -137,8 +138,19 @@ struct flb_kafka *flb_kafka_conf_create(struct flb_output_instance *ins,
         ctx->message_key_len = strlen(tmp);
     }
     else {
-        ctx->timestamp_key = NULL;
-        ctx->timestamp_key_len = 0;
+        ctx->message_key = NULL;
+        ctx->message_key_len = 0;
+    }
+
+    /* Config: Message_Key_Field */
+    tmp = flb_output_get_property("message_key_field", ins);
+    if (tmp) {
+        ctx->message_key_field = flb_strdup(tmp);
+        ctx->message_key_field_len = strlen(tmp);
+    }
+    else {
+        ctx->message_key_field = NULL;
+        ctx->message_key_field_len = 0;
     }
 
     /* Config: Timestamp_Key */
@@ -253,6 +265,10 @@ int flb_kafka_conf_destroy(struct flb_kafka *ctx)
     if (ctx->message_key) {
         flb_free(ctx->message_key);
     }
+    
+    if (ctx->message_key_field) {
+        flb_free(ctx->message_key_field);
+    }
 
     flb_sds_destroy(ctx->gelf_fields.timestamp_key);
     flb_sds_destroy(ctx->gelf_fields.host_key);
