@@ -33,6 +33,7 @@
 #include "rdkafka_partition.h"
 #include "rdkafka_interceptor.h"
 #include "rdkafka_header.h"
+#include "rdkafka_idempotence.h"
 #include "rdcrc32.h"
 #include "rdmurmur2.h"
 #include "rdrand.h"
@@ -107,6 +108,7 @@ rd_kafka_msg_t *rd_kafka_msg_new00 (rd_kafka_itopic_t *rkt,
         rkm->rkm_offset     = RD_KAFKA_OFFSET_INVALID;
 	rkm->rkm_timestamp  = 0;
 	rkm->rkm_tstype     = RD_KAFKA_TIMESTAMP_NOT_AVAILABLE;
+        rkm->rkm_status     = RD_KAFKA_MSG_STATUS_NOT_PERSISTED;
         rkm->rkm_headers    = NULL;
 
 	p = (char *)(rkm+1);
@@ -214,7 +216,7 @@ static rd_kafka_msg_t *rd_kafka_msg_new0 (rd_kafka_itopic_t *rkt,
 		rkm->rkm_ts_timeout = INT64_MAX;
 	} else {
 		rkm->rkm_ts_timeout = now +
-			rkt->rkt_conf.message_timeout_ms * 1000;
+			(int64_t) rkt->rkt_conf.message_timeout_ms * 1000;
 	}
 
         /* Call interceptor chain for on_send */
@@ -245,6 +247,11 @@ int rd_kafka_msg_new (rd_kafka_itopic_t *rkt, int32_t force_partition,
 	rd_kafka_resp_err_t err;
 	int errnox;
 
+        if (unlikely((err = rd_kafka_fatal_error_code(rkt->rkt_rk)))) {
+                rd_kafka_set_last_error(err, ECANCELED);
+                return -1;
+        }
+
         /* Create message */
         rkm = rd_kafka_msg_new0(rkt, force_partition, msgflags,
                                 payload, len, key, keylen, msg_opaque,
@@ -301,10 +308,13 @@ rd_kafka_resp_err_t rd_kafka_producev (rd_kafka_t *rk, ...) {
         rd_kafka_topic_t *app_rkt;
         shptr_rd_kafka_itopic_t *s_rkt = NULL;
         rd_kafka_itopic_t *rkt;
-        rd_kafka_resp_err_t err = RD_KAFKA_RESP_ERR_NO_ERROR;
+        rd_kafka_resp_err_t err;
         rd_kafka_headers_t *hdrs = NULL;
         rd_kafka_headers_t *app_hdrs = NULL; /* App-provided headers list */
 
+        if (unlikely((err = rd_kafka_fatal_error_code(rk))))
+                return err;
+
         va_start(ap, rk);
         while (!err &&
                (vtype = va_arg(ap, rd_kafka_vtype_t)) != RD_KAFKA_VTYPE_END) {
@@ -441,6 +451,25 @@ rd_kafka_resp_err_t rd_kafka_producev (rd_kafka_t *rk, ...) {
         return err;
 }
 
+
+
+/**
+ * @brief Produce a single message.
+ * @locality any application thread
+ * @locks none
+ */
+int rd_kafka_produce (rd_kafka_topic_t *rkt, int32_t partition,
+                      int msgflags,
+                      void *payload, size_t len,
+                      const void *key, size_t keylen,
+                      void *msg_opaque) {
+        return rd_kafka_msg_new(rd_kafka_topic_a2i(rkt), partition,
+                                msgflags, payload, len,
+                                key, keylen, msg_opaque);
+}
+
+
+
 /**
  * Produce a batch of messages.
  * Returns the number of messages succesfully queued for producing.
@@ -456,19 +485,19 @@ int rd_kafka_produce_batch (rd_kafka_topic_t *app_rkt, int32_t partition,
         int good = 0;
         int multiple_partitions = (partition == RD_KAFKA_PARTITION_UA ||
                                    (msgflags & RD_KAFKA_MSG_F_PARTITION));
-        rd_kafka_resp_err_t all_err = 0;
+        rd_kafka_resp_err_t all_err;
         rd_kafka_itopic_t *rkt = rd_kafka_topic_a2i(app_rkt);
-        rd_kafka_toppar_t *rktp = NULL;
         shptr_rd_kafka_toppar_t *s_rktp = NULL;
 
-        /* For multiple partitions; hold lock for entire run,
-         * for one partition: only acquire for now. */
+        /* Propagated per-message below */
+        all_err = rd_kafka_fatal_error_code(rkt->rkt_rk);
+
         rd_kafka_topic_rdlock(rkt);
         if (!multiple_partitions) {
+                /* Single partition: look up the rktp once. */
                 s_rktp = rd_kafka_toppar_get_avail(rkt, partition,
                                                    1/*ua on miss*/, &all_err);
-                rktp = rd_kafka_toppar_s2i(s_rktp);
-                rd_kafka_topic_rdunlock(rkt);
+
         } else {
                 /* Indicate to lower-level msg_new..() that rkt is locked
                  * so that they may unlock it momentarily if blocking. */
@@ -519,14 +548,20 @@ int rd_kafka_produce_batch (rd_kafka_topic_t *app_rkt, int32_t partition,
                                     rkm->rkm_partition !=
                                     rd_kafka_toppar_s2i(s_rktp)->
                                     rktp_partition) {
+                                        rd_kafka_resp_err_t err;
                                         if (s_rktp != NULL)
                                                 rd_kafka_toppar_destroy(s_rktp);
                                         s_rktp = rd_kafka_toppar_get_avail(
                                                 rkt, rkm->rkm_partition,
-                                                1/*ua on miss*/, &all_err);
+                                                1/*ua on miss*/, &err);
+
+                                        if (unlikely(!s_rktp)) {
+                                                rkmessages[i].err = err;
+                                                continue;
+                                        }
                                 }
-                                rktp = rd_kafka_toppar_s2i(s_rktp);
-                                rd_kafka_toppar_enq_msg(rktp, rkm);
+                                rd_kafka_toppar_enq_msg(
+                                        rd_kafka_toppar_s2i(s_rktp), rkm);
                         }
 
                         if (unlikely(rkmessages[i].err)) {
@@ -541,15 +576,15 @@ int rd_kafka_produce_batch (rd_kafka_topic_t *app_rkt, int32_t partition,
 
                 } else {
                         /* Single destination partition. */
-                        rd_kafka_toppar_enq_msg(rktp, rkm);
+                        rd_kafka_toppar_enq_msg(rd_kafka_toppar_s2i(s_rktp),
+                                                rkm);
                 }
 
                 rkmessages[i].err = RD_KAFKA_RESP_ERR_NO_ERROR;
                 good++;
         }
 
-        if (multiple_partitions)
-                rd_kafka_topic_rdunlock(rkt);
+        rd_kafka_topic_rdunlock(rkt);
         if (s_rktp != NULL)
                 rd_kafka_toppar_destroy(s_rktp);
 
@@ -557,32 +592,50 @@ int rd_kafka_produce_batch (rd_kafka_topic_t *app_rkt, int32_t partition,
 }
 
 /**
- * Scan 'rkmq' for messages that have timed out and remove them from
- * 'rkmq' and add to 'timedout'.
+ * @brief Scan \p rkmq for messages that have timed out and remove them from
+ *        \p rkmq and add to \p timedout queue.
  *
- * Returns the number of messages timed out.
+ * @param abs_next_timeout will be set to the next message timeout, or 0
+ *                         if no timeout. Optional, may be NULL.
+ *
+ * @returns the number of messages timed out.
+ *
+ * @locality any
+ * @locks toppar_lock MUST be held
  */
-int rd_kafka_msgq_age_scan (rd_kafka_msgq_t *rkmq,
-			    rd_kafka_msgq_t *timedout,
-			    rd_ts_t now) {
-	rd_kafka_msg_t *rkm, *tmp;
-	int cnt = timedout->rkmq_msg_cnt;
+int rd_kafka_msgq_age_scan (rd_kafka_toppar_t *rktp,
+                            rd_kafka_msgq_t *rkmq,
+                            rd_kafka_msgq_t *timedout,
+                            rd_ts_t now,
+                            rd_ts_t *abs_next_timeout) {
+        rd_kafka_msg_t *rkm, *tmp, *first = NULL;
+        int cnt = timedout->rkmq_msg_cnt;
 
-	/* Assume messages are added in time sequencial order */
-	TAILQ_FOREACH_SAFE(rkm, &rkmq->rkmq_msgs, rkm_link, tmp) {
-                /* FIXME: this is no longer true */
-		if (likely(rkm->rkm_ts_timeout > now))
-			break;
+        if (abs_next_timeout)
+                *abs_next_timeout = 0;
 
-		rd_kafka_msgq_deq(rkmq, rkm, 1);
-		rd_kafka_msgq_enq(timedout, rkm);
-	}
+        /* Assume messages are added in time sequencial order */
+        TAILQ_FOREACH_SAFE(rkm, &rkmq->rkmq_msgs, rkm_link, tmp) {
+                /* NOTE: this is not true for the deprecated (and soon removed)
+                 *       LIFO queuing strategy. */
+                if (likely(rkm->rkm_ts_timeout > now)) {
+                        if (abs_next_timeout)
+                                *abs_next_timeout = rkm->rkm_ts_timeout;
+                        break;
+                }
 
-	return timedout->rkmq_msg_cnt - cnt;
+                if (!first)
+                        first = rkm;
+
+                rd_kafka_msgq_deq(rkmq, rkm, 1);
+                rd_kafka_msgq_enq(timedout, rkm);
+        }
+
+        return timedout->rkmq_msg_cnt - cnt;
 }
 
 
-static RD_INLINE int
+int
 rd_kafka_msgq_enq_sorted0 (rd_kafka_msgq_t *rkmq,
                            rd_kafka_msg_t *rkm,
                            int (*order_cmp) (const void *, const void *)) {
@@ -595,7 +648,7 @@ rd_kafka_msgq_enq_sorted0 (rd_kafka_msgq_t *rkmq,
 int rd_kafka_msgq_enq_sorted (const rd_kafka_itopic_t *rkt,
                               rd_kafka_msgq_t *rkmq,
                               rd_kafka_msg_t *rkm) {
-        rd_dassert(rkm->rkm_u.producer.msgseq != 0);
+        rd_dassert(rkm->rkm_u.producer.msgid != 0);
         return rd_kafka_msgq_enq_sorted0(rkmq, rkm,
                                          rkt->rkt_conf.msg_order_cmp);
 }
@@ -623,6 +676,57 @@ rd_kafka_msg_t *rd_kafka_msgq_find_pos (const rd_kafka_msgq_t *rkmq,
 }
 
 
+/**
+ * @brief Set per-message metadata for all messages in \p rkmq
+ */
+void rd_kafka_msgq_set_metadata (rd_kafka_msgq_t *rkmq,
+                                 int64_t base_offset, int64_t timestamp,
+                                 rd_kafka_msg_status_t status) {
+        rd_kafka_msg_t *rkm;
+
+        TAILQ_FOREACH(rkm, &rkmq->rkmq_msgs, rkm_link) {
+                rkm->rkm_offset = base_offset++;
+                if (timestamp != -1) {
+                        rkm->rkm_timestamp = timestamp;
+                        rkm->rkm_tstype = RD_KAFKA_MSG_ATTR_LOG_APPEND_TIME;
+                }
+
+                /* Don't downgrade a message from any form of PERSISTED
+                 * to NOT_PERSISTED, since the original cause of indicating
+                 * PERSISTED can't be changed.
+                 * E.g., a previous ack or in-flight timeout. */
+                if (unlikely(status == RD_KAFKA_MSG_STATUS_NOT_PERSISTED &&
+                             rkm->rkm_status != RD_KAFKA_MSG_STATUS_NOT_PERSISTED))
+                        continue;
+
+                rkm->rkm_status = status;
+        }
+}
+
+
+/**
+ * @brief Move all messages in \p src to \p dst whose msgid <= last_msgid.
+ *
+ * @remark src must be ordered
+ */
+void rd_kafka_msgq_move_acked (rd_kafka_msgq_t *dest, rd_kafka_msgq_t *src,
+                               uint64_t last_msgid,
+                               rd_kafka_msg_status_t status) {
+        rd_kafka_msg_t *rkm;
+
+        while ((rkm = rd_kafka_msgq_first(src)) &&
+               rkm->rkm_u.producer.msgid <= last_msgid) {
+                rd_kafka_msgq_deq(src, rkm, 1);
+		rd_kafka_msgq_enq(dest, rkm);
+
+                rkm->rkm_status = status;
+        }
+
+        rd_kafka_msgq_verify_order(NULL, dest, 0, rd_false);
+        rd_kafka_msgq_verify_order(NULL, src, 0, rd_false);
+}
+
+
 
 int32_t rd_kafka_msg_partitioner_random (const rd_kafka_topic_t *rkt,
 					 const void *key, size_t keylen,
@@ -1085,6 +1189,16 @@ void rd_kafka_message_set_headers (rd_kafka_message_t *rkmessage,
 
 
 
+rd_kafka_msg_status_t
+rd_kafka_message_status (const rd_kafka_message_t *rkmessage) {
+        rd_kafka_msg_t *rkm;
+
+        rkm = rd_kafka_message2msg((rd_kafka_message_t *)rkmessage);
+
+        return rkm->rkm_status;
+}
+
+
 void rd_kafka_msgq_dump (FILE *fp, const char *what, rd_kafka_msgq_t *rkmq) {
         rd_kafka_msg_t *rkm;
 
@@ -1092,13 +1206,169 @@ void rd_kafka_msgq_dump (FILE *fp, const char *what, rd_kafka_msgq_t *rkmq) {
                 rd_kafka_msgq_len(rkmq), rd_kafka_msgq_size(rkmq));
         TAILQ_FOREACH(rkm, &rkmq->rkmq_msgs, rkm_link) {
                 fprintf(fp, " [%"PRId32"]@%"PRId64
-                        ": rkm msgseq %"PRIu64": \"%.*s\"\n",
+                        ": rkm msgid %"PRIu64": \"%.*s\"\n",
                         rkm->rkm_partition, rkm->rkm_offset,
-                        rkm->rkm_u.producer.msgseq,
+                        rkm->rkm_u.producer.msgid,
                         (int)rkm->rkm_len, (const char *)rkm->rkm_payload);
         }
 }
 
+
+
+
+/**
+ * @brief Destroy resources associated with msgbatch
+ */
+void rd_kafka_msgbatch_destroy (rd_kafka_msgbatch_t *rkmb) {
+        if (rkmb->s_rktp) {
+                rd_kafka_toppar_destroy(rkmb->s_rktp);
+                rkmb->s_rktp = NULL;
+        }
+
+        rd_assert(RD_KAFKA_MSGQ_EMPTY(&rkmb->msgq));
+}
+
+
+/**
+ * @brief Initialize a message batch for the Idempotent Producer.
+ *
+ * @param rkm is the first message in the batch.
+ */
+void rd_kafka_msgbatch_init (rd_kafka_msgbatch_t *rkmb,
+                             rd_kafka_toppar_t *rktp,
+                             rd_kafka_pid_t pid) {
+        memset(rkmb, 0, sizeof(*rkmb));
+
+        rkmb->s_rktp = rd_kafka_toppar_keep(rktp);
+
+        rd_kafka_msgq_init(&rkmb->msgq);
+
+        rkmb->pid = pid;
+        rkmb->first_seq = -1;
+}
+
+
+/**
+ * @brief Set the first message in the batch. which is used to set
+ *        the BaseSequence and keep track of batch reconstruction range.
+ */
+void rd_kafka_msgbatch_set_first_msg (rd_kafka_msgbatch_t *rkmb,
+                                      rd_kafka_msg_t *rkm) {
+        rd_assert(rkmb->first_msgid == 0);
+
+        if (!rd_kafka_pid_valid(rkmb->pid))
+                return;
+
+        rkmb->first_msgid = rkm->rkm_u.producer.msgid;
+
+        /* Our msgid counter is 64-bits, but the
+         * Kafka protocol's sequence is only 31 (signed), so we'll
+         * need to handle wrapping. */
+        rkmb->first_seq =
+                rd_kafka_seq_wrap(rkm->rkm_u.producer.msgid -
+                                  rd_kafka_toppar_s2i(rkmb->s_rktp)->
+                                  rktp_eos.epoch_base_msgid);
+
+        /* Check if there is a stored last message
+         * on the first msg, which means an entire
+         * batch of messages are being retried and
+         * we need to maintain the exact messages
+         * of the original batch.
+         * Simply tracking the last message, on
+         * the first message, is sufficient for now.
+         * Will be 0 if not applicable. */
+        rkmb->last_msgid = rkm->rkm_u.producer.last_msgid;
+}
+
+
+
+/**
+ * @brief Message batch is ready to be transmitted.
+ *
+ * @remark This function assumes the batch will be transmitted and increases
+ *         the toppar's in-flight count.
+ */
+void rd_kafka_msgbatch_ready_produce (rd_kafka_msgbatch_t *rkmb) {
+        rd_kafka_toppar_t *rktp = rd_kafka_toppar_s2i(rkmb->s_rktp);
+        rd_kafka_t *rk = rktp->rktp_rkt->rkt_rk;
+
+        /* Keep track of number of requests in-flight per partition,
+         * and the number of partitions with in-flight requests when
+         * idempotent producer - this is used to drain partitions
+         * before resetting the PID. */
+        if (rd_atomic32_add(&rktp->rktp_msgs_inflight,
+                            rd_kafka_msgq_len(&rkmb->msgq)) ==
+            rd_kafka_msgq_len(&rkmb->msgq) &&
+            rd_kafka_is_idempotent(rk))
+                rd_kafka_idemp_inflight_toppar_add(rk, rktp);
+}
+
+
+/**
+ * @brief Verify order (by msgid) in message queue.
+ *        For development use only.
+ */
+void rd_kafka_msgq_verify_order0 (const char *function, int line,
+                                  const rd_kafka_toppar_t *rktp,
+                                  const rd_kafka_msgq_t *rkmq,
+                                  uint64_t exp_first_msgid,
+                                  rd_bool_t gapless) {
+        const rd_kafka_msg_t *rkm;
+        uint64_t exp;
+        int errcnt = 0;
+        int cnt = 0;
+        const char *topic = rktp ? rktp->rktp_rkt->rkt_topic->str : "n/a";
+        int32_t partition = rktp ? rktp->rktp_partition : -1;
+
+        if (rd_kafka_msgq_len(rkmq) == 0)
+                return;
+
+        if (exp_first_msgid)
+                exp = exp_first_msgid;
+        else {
+                exp = rd_kafka_msgq_first(rkmq)->rkm_u.producer.msgid;
+                if (exp == 0) /* message without msgid (e.g., UA partition) */
+                        return;
+        }
+
+        TAILQ_FOREACH(rkm, &rkmq->rkmq_msgs, rkm_link) {
+#if 0
+                printf("%s:%d: %s [%"PRId32"]: rkm #%d (%p) "
+                       "msgid %"PRIu64"\n",
+                       function, line,
+                       topic, partition,
+                       cnt, rkm, rkm->rkm_u.producer.msgid);
+#endif
+                if (gapless &&
+                    rkm->rkm_u.producer.msgid != exp) {
+                        printf("%s:%d: %s [%"PRId32"]: rkm #%d (%p) "
+                               "msgid %"PRIu64": "
+                               "expected msgid %"PRIu64"\n",
+                               function, line,
+                               topic, partition,
+                               cnt, rkm, rkm->rkm_u.producer.msgid,
+                               exp);
+                        errcnt++;
+                } else if (!gapless && rkm->rkm_u.producer.msgid < exp) {
+                        printf("%s:%d: %s [%"PRId32"]: rkm #%d (%p) "
+                               "msgid %"PRIu64": "
+                               "expected increased msgid >= %"PRIu64"\n",
+                               function, line,
+                               topic, partition,
+                               cnt, rkm, rkm->rkm_u.producer.msgid,
+                               exp);
+                        errcnt++;
+                } else
+                        exp++;
+
+                cnt++;
+        }
+
+        rd_assert(!errcnt);
+}
+
+
+
 /**
  * @name Unit tests
  */
@@ -1106,7 +1376,7 @@ void rd_kafka_msgq_dump (FILE *fp, const char *what, rd_kafka_msgq_t *rkmq) {
 /**
  * @brief Unittest: message allocator
  */
-static rd_kafka_msg_t *ut_rd_kafka_msg_new (void) {
+rd_kafka_msg_t *ut_rd_kafka_msg_new (void) {
         rd_kafka_msg_t *rkm;
 
         rkm = rd_calloc(1, sizeof(*rkm));
@@ -1122,7 +1392,7 @@ static rd_kafka_msg_t *ut_rd_kafka_msg_new (void) {
 /**
  * @brief Unittest: destroy all messages in queue
  */
-static void ut_rd_kafka_msgq_purge (rd_kafka_msgq_t *rkmq) {
+void ut_rd_kafka_msgq_purge (rd_kafka_msgq_t *rkmq) {
         rd_kafka_msg_t *rkm, *tmp;
 
         TAILQ_FOREACH_SAFE(rkm, &rkmq->rkmq_msgs, rkm_link, tmp)
@@ -1144,11 +1414,11 @@ static int ut_verify_msgq_order (const char *what,
         int cnt = 0;
 
         TAILQ_FOREACH(rkm, &rkmq->rkmq_msgs, rkm_link) {
-                if (rkm->rkm_u.producer.msgseq != expected) {
-                        RD_UT_SAY("%s: expected msgseq %"PRIu64
+                if (rkm->rkm_u.producer.msgid != expected) {
+                        RD_UT_SAY("%s: expected msgid %"PRIu64
                                   " not %"PRIu64" at index #%d",
                                   what, expected,
-                                  rkm->rkm_u.producer.msgseq, cnt);
+                                  rkm->rkm_u.producer.msgid, cnt);
                         fails++;
                 }
                 cnt++;
@@ -1166,14 +1436,14 @@ static int unittest_msgq_order (const char *what, int fifo,
                           int (*cmp) (const void *, const void *)) {
         rd_kafka_msgq_t rkmq = RD_KAFKA_MSGQ_INITIALIZER(rkmq);
         rd_kafka_msg_t *rkm;
-        rd_kafka_msgq_t sendq;
+        rd_kafka_msgq_t sendq, sendq2;
         int i;
 
         RD_UT_SAY("%s: testing in %s mode", what, fifo? "FIFO" : "LIFO");
 
         for (i = 1 ; i <= 6 ; i++) {
                 rkm = ut_rd_kafka_msg_new();
-                rkm->rkm_u.producer.msgseq = i;
+                rkm->rkm_u.producer.msgid = i;
                 rd_kafka_msgq_enq_sorted0(&rkmq, rkm, cmp);
         }
 
@@ -1207,7 +1477,8 @@ static int unittest_msgq_order (const char *what, int fifo,
 
         /* Retry the messages, which moves them back to sendq
          * maintaining the original order */
-        rd_kafka_retry_msgq(&rkmq, &sendq, 1, 1, 0, cmp);
+        rd_kafka_retry_msgq(&rkmq, &sendq, 1, 1, 0,
+                            RD_KAFKA_MSG_STATUS_NOT_PERSISTED, cmp);
 
         RD_UT_ASSERT(rd_kafka_msgq_len(&sendq) == 0,
                      "sendq FIFO should be empty, not contain %d messages",
@@ -1244,7 +1515,8 @@ static int unittest_msgq_order (const char *what, int fifo,
 
         /* Retry the messages, which should now keep the 3 first messages
          * on sendq (no more retries) and just number 4 moved back. */
-        rd_kafka_retry_msgq(&rkmq, &sendq, 1, 1, 0, cmp);
+        rd_kafka_retry_msgq(&rkmq, &sendq, 1, 1, 0,
+                            RD_KAFKA_MSG_STATUS_NOT_PERSISTED, cmp);
 
         if (fifo) {
                 if (ut_verify_msgq_order("readded #2", &rkmq, 4, 6))
@@ -1261,19 +1533,98 @@ static int unittest_msgq_order (const char *what, int fifo,
                         return 1;
         }
 
+        /* Move all messages back on rkmq */
+        rd_kafka_retry_msgq(&rkmq, &sendq, 0, 1000, 0,
+                            RD_KAFKA_MSG_STATUS_NOT_PERSISTED, cmp);
+
+
+        /* Move first half of messages to sendq (1,2,3).
+         * Move second half o messages to sendq2 (4,5,6).
+         * Add new message to rkmq (7).
+         * Move first half of messages back on rkmq (1,2,3,7).
+         * Move second half back on the rkmq (1,2,3,4,5,6,7). */
+        rd_kafka_msgq_init(&sendq);
+        rd_kafka_msgq_init(&sendq2);
+
+        while (rd_kafka_msgq_len(&sendq) < 3)
+                rd_kafka_msgq_enq(&sendq, rd_kafka_msgq_pop(&rkmq));
+
+        while (rd_kafka_msgq_len(&sendq2) < 3)
+                rd_kafka_msgq_enq(&sendq2, rd_kafka_msgq_pop(&rkmq));
+
+        rkm = ut_rd_kafka_msg_new();
+        rkm->rkm_u.producer.msgid = i;
+        rd_kafka_msgq_enq_sorted0(&rkmq, rkm, cmp);
+
+        rd_kafka_retry_msgq(&rkmq, &sendq, 0, 1000, 0,
+                            RD_KAFKA_MSG_STATUS_NOT_PERSISTED, cmp);
+        rd_kafka_retry_msgq(&rkmq, &sendq2, 0, 1000, 0,
+                            RD_KAFKA_MSG_STATUS_NOT_PERSISTED, cmp);
+
+        RD_UT_ASSERT(rd_kafka_msgq_len(&sendq) == 0,
+                     "sendq FIFO should be empty, not contain %d messages",
+                     rd_kafka_msgq_len(&sendq));
+        RD_UT_ASSERT(rd_kafka_msgq_len(&sendq2) == 0,
+                     "sendq2 FIFO should be empty, not contain %d messages",
+                     rd_kafka_msgq_len(&sendq2));
+
+        if (fifo) {
+                if (ut_verify_msgq_order("inject", &rkmq, 1, 7))
+                        return 1;
+        } else {
+                if (ut_verify_msgq_order("readded #2", &rkmq, 7, 1))
+                        return 1;
+        }
+
+
         ut_rd_kafka_msgq_purge(&sendq);
+        ut_rd_kafka_msgq_purge(&sendq2);
         ut_rd_kafka_msgq_purge(&rkmq);
 
         return 0;
 
 }
 
+/**
+ * @brief Verify that rd_kafka_seq_wrap() works.
+ */
+static int unittest_msg_seq_wrap (void) {
+        static const struct exp {
+                int64_t in;
+                int32_t out;
+        } exp[] = {
+                { 0,  0 },
+                { 1, 1 },
+                { (int64_t)INT32_MAX+2, 1 },
+                { (int64_t)INT32_MAX+1, 0 },
+                { INT32_MAX, INT32_MAX },
+                { INT32_MAX-1, INT32_MAX-1 },
+                { INT32_MAX-2, INT32_MAX-2 },
+                { ((int64_t)1<<33)-2, INT32_MAX-1 },
+                { ((int64_t)1<<33)-1, INT32_MAX },
+                { ((int64_t)1<<34), 0 },
+                { ((int64_t)1<<35)+3, 3 },
+                { 1710+1229, 2939 },
+                { -1, -1 },
+        };
+        int i;
+
+        for (i = 0 ; exp[i].in != -1 ; i++) {
+                int32_t wseq = rd_kafka_seq_wrap(exp[i].in);
+                RD_UT_ASSERT(wseq == exp[i].out,
+                             "Expected seq_wrap(%"PRId64") -> %"PRId32
+                             ", not %"PRId32,
+                             exp[i].in, exp[i].out, wseq);
+        }
+
+        RD_UT_PASS();
+}
 
 int unittest_msg (void) {
         int fails = 0;
 
-        fails += unittest_msgq_order("FIFO", 1, rd_kafka_msg_cmp_msgseq);
-        fails += unittest_msgq_order("LIFO", 0, rd_kafka_msg_cmp_msgseq_lifo);
+        fails += unittest_msgq_order("FIFO", 1, rd_kafka_msg_cmp_msgid);
+        fails += unittest_msg_seq_wrap();
 
         return fails;
 }
