@@ -37,12 +37,7 @@ int cb_nats_init(struct flb_output_instance *ins, struct flb_config *config,
     struct flb_out_nats_config *ctx;
 
     /* Set default network configuration */
-    if (!ins->host.name) {
-        ins->host.name = flb_strdup("127.0.0.1");
-    }
-    if (ins->host.port == 0) {
-        ins->host.port = 4222;
-    }
+    flb_output_net_default("127.0.0.1", 4222, ins);
 
     /* Allocate plugin context */
     ctx = flb_malloc(sizeof(struct flb_out_nats_config));
@@ -188,6 +183,13 @@ void cb_nats_flush(const void *data, size_t bytes,
 
     /* Compose the NATS Publish request */
     request = flb_malloc(json_len + tag_len + 32);
+    if (!request) {
+        flb_errno();
+        flb_sds_destroy(json_msg);
+        flb_upstream_conn_release(u_conn);
+        FLB_OUTPUT_RETURN(FLB_RETRY);
+    }
+
     req_len = snprintf(request, tag_len + 32, "PUB %s %zu\r\n",
                        tag, json_len);
 
