@@ -2,6 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
+ *  Copyright (C) 2019      The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -43,7 +44,7 @@ static int cb_splunk_init(struct flb_output_instance *ins,
     return 0;
 }
 
-int splunk_format(void *in_buf, size_t in_bytes,
+int splunk_format(const void *in_buf, size_t in_bytes,
                   char **out_buf, size_t *out_size,
                   struct flb_splunk *ctx)
 {
@@ -73,7 +74,7 @@ int splunk_format(void *in_buf, size_t in_bytes,
     /* Iterate the original buffer and perform adjustments */
     msgpack_unpacked_init(&result);
 
-    while (msgpack_unpack_next(&result, in_buf, in_bytes, &off)) {
+    while (msgpack_unpack_next(&result, in_buf, in_bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
         root = result.data;
 
         /* Get timestamp */
@@ -87,7 +88,11 @@ int splunk_format(void *in_buf, size_t in_bytes,
         map = root.via.array.ptr[1];
         map_size = map.via.map.size;
 
-        msgpack_pack_map(&mp_pck, 2);
+        if (ctx->splunk_send_raw == FLB_TRUE) {
+            msgpack_pack_map(&mp_pck, 1 + map_size /* time + all k/v */);
+        } else {
+            msgpack_pack_map(&mp_pck, 2 /* time + event */);
+        }
 
         /* Append the time key */
         msgpack_pack_str(&mp_pck, sizeof(FLB_SPLUNK_DEFAULT_TIME) -1);
@@ -96,12 +101,14 @@ int splunk_format(void *in_buf, size_t in_bytes,
                               sizeof(FLB_SPLUNK_DEFAULT_TIME) - 1);
         msgpack_pack_double(&mp_pck, t);
 
-        /* Append event key */
-        msgpack_pack_str(&mp_pck, sizeof(FLB_SPLUNK_DEFAULT_EVENT) -1);
-        msgpack_pack_str_body(&mp_pck,
-                              FLB_SPLUNK_DEFAULT_EVENT,
-                              sizeof(FLB_SPLUNK_DEFAULT_EVENT) - 1);
-        msgpack_pack_map(&mp_pck, map_size);
+        if (ctx->splunk_send_raw == FLB_FALSE) {
+            /* Add k/v pairs under the key 'event' instead of to the top level object */
+            msgpack_pack_str(&mp_pck, sizeof(FLB_SPLUNK_DEFAULT_EVENT) -1);
+            msgpack_pack_str_body(&mp_pck,
+                                  FLB_SPLUNK_DEFAULT_EVENT,
+                                  sizeof(FLB_SPLUNK_DEFAULT_EVENT) - 1);
+            msgpack_pack_map(&mp_pck, map_size);
+        }
 
         /* Append k/v */
         for (i = 0; i < map_size; i++) {
@@ -142,8 +149,8 @@ int splunk_format(void *in_buf, size_t in_bytes,
     return 0;
 }
 
-static void cb_splunk_flush(void *data, size_t bytes,
-                            char *tag, int tag_len,
+static void cb_splunk_flush(const void *data, size_t bytes,
+                            const char *tag, int tag_len,
                             struct flb_input_instance *i_ins,
                             void *out_context,
                             struct flb_config *config)
