@@ -279,6 +279,11 @@ struct flb_stackdriver *flb_stackdriver_conf_create(struct flb_output_instance *
         ctx->resource = flb_sds_create(FLB_SDS_RESOURCE_TYPE);
     }
 
+    tmp = flb_output_get_property("severity_key", ins);
+    if (tmp) {
+        ctx->severity_key = flb_sds_create(tmp);
+    }
+
     return ctx;
 }
 
@@ -298,6 +303,7 @@ int flb_stackdriver_conf_destroy(struct flb_stackdriver *ctx)
     flb_sds_destroy(ctx->auth_uri);
     flb_sds_destroy(ctx->token_uri);
     flb_sds_destroy(ctx->resource);
+    flb_sds_destroy(ctx->severity_key);
 
     if (ctx->o) {
         flb_oauth2_destroy(ctx->o);
