@@ -27,6 +27,7 @@
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_mem.h>
 #include <fluent-bit/flb_str.h>
+#include <fluent-bit/flb_kv.h>
 #include <fluent-bit/flb_env.h>
 #include <fluent-bit/flb_macros.h>
 #include <fluent-bit/flb_config.h>
@@ -40,6 +41,7 @@
 #include <fluent-bit/flb_scheduler.h>
 #include <fluent-bit/flb_http_server.h>
 #include <fluent-bit/flb_plugin.h>
+#include <fluent-bit/flb_utils.h>
 
 int flb_regex_init();
 
@@ -135,7 +137,6 @@ struct flb_config *flb_config_init()
 
     /* Flush */
     config->flush        = FLB_CONFIG_FLUSH_SECS;
-    config->flush_method = FLB_FLUSH_LIBCO;
     config->daemon       = FLB_FALSE;
     config->init_time    = time(NULL);
     config->kernel       = flb_kernel_info();
@@ -312,13 +313,15 @@ void flb_config_exit(struct flb_config *config)
     }
 #endif
 
-#ifdef FLB_HAVE_STATS
-    flb_stats_exit(config);
-#endif
-
     if (config->storage_path) {
         flb_free(config->storage_path);
     }
+    if (config->storage_sync) {
+        flb_free(config->storage_sync);
+    }
+    if (config->storage_bl_mem_limit) {
+        flb_free(config->storage_bl_mem_limit);
+    }
 
 #ifdef FLB_HAVE_STREAM_PROCESSOR
     if (config->stream_processor_file) {
@@ -336,17 +339,7 @@ void flb_config_exit(struct flb_config *config)
 
 const char *flb_config_prop_get(const char *key, struct mk_list *list)
 {
-    struct mk_list *head;
-    struct flb_config_prop *p;
-
-    mk_list_foreach(head, list) {
-        p = mk_list_entry(head, struct flb_config_prop, _head);
-        if (strcasecmp(key, p->key) == 0) {
-            return p->val;
-        }
-    }
-
-    return NULL;
+    return flb_kv_get_key_value(key, list);
 }
 
 static inline int prop_key_check(const char *key, const char *kv, int k_len)
@@ -388,15 +381,6 @@ static int set_log_level(struct flb_config *config, const char *v_str)
     return 0;
 }
 
-static inline int atobool(const char *v)
-{
-    return  (strcasecmp("true", v) == 0 ||
-             strcasecmp("on", v) == 0 ||
-             strcasecmp("yes", v) == 0)
-        ? FLB_TRUE
-        : FLB_FALSE;
-}
-
 int flb_config_set_property(struct flb_config *config,
                             const char *k, const char *v)
 {
@@ -407,7 +391,7 @@ int flb_config_set_property(struct flb_config *config,
     char **s_val;
     size_t len = strnlen(k, 256);
     char *key = service_configs[0].key;
-    char *tmp = NULL;
+    flb_sds_t tmp = NULL;
 
     while (key != NULL) {
         if (prop_key_check(key, k,len) == 0) {
@@ -415,7 +399,7 @@ int flb_config_set_property(struct flb_config *config,
                 tmp = flb_env_var_translate(config->env, v);
                 if (tmp) {
                     ret = set_log_level(config, tmp);
-                    flb_free(tmp);
+                    flb_sds_destroy(tmp);
                     tmp = NULL;
                 }
                 else {
@@ -426,14 +410,14 @@ int flb_config_set_property(struct flb_config *config,
 #ifdef FLB_HAVE_PARSER
                 tmp = flb_env_var_translate(config->env, v);
                 ret = flb_parser_conf_file(tmp, config);
-                flb_free(tmp);
+                flb_sds_destroy(tmp);
                 tmp = NULL;
 #endif
             }
             else if (!strncasecmp(key, FLB_CONF_STR_PLUGINS_FILE, 32)) {
                 tmp = flb_env_var_translate(config->env, v);
                 ret = flb_plugin_load_config_file(tmp, config);
-                flb_free(tmp);
+                flb_sds_destroy(tmp);
                 tmp = NULL;
             }
             else {
@@ -443,24 +427,26 @@ int flb_config_set_property(struct flb_config *config,
                 case FLB_CONF_TYPE_INT:
                     i_val  = (int*)((char*)config + service_configs[i].offset);
                     *i_val = atoi(tmp);
-                    flb_free(tmp);
+                    flb_sds_destroy(tmp);
                     break;
                 case FLB_CONF_TYPE_DOUBLE:
                     d_val  = (double*)((char*)config + service_configs[i].offset);
                     *d_val = atof(tmp);
-                    flb_free(tmp);
+                    flb_sds_destroy(tmp);
                     break;
                 case FLB_CONF_TYPE_BOOL:
                     i_val = (int*)((char*)config+service_configs[i].offset);
-                    *i_val = atobool(tmp);
-                    flb_free(tmp);
+                    *i_val = flb_utils_bool(tmp);
+                    flb_sds_destroy(tmp);
                     break;
                 case FLB_CONF_TYPE_STR:
                     s_val = (char**)((char*)config+service_configs[i].offset);
                     if ( *s_val != NULL ) {
                         flb_free(*s_val); /* release before overwriting */
                     }
-                    *s_val = tmp;
+
+                    *s_val = flb_strdup(tmp);
+                    flb_sds_destroy(tmp);
                     break;
                 default:
                     ret = -1;
@@ -469,7 +455,7 @@ int flb_config_set_property(struct flb_config *config,
 
             if (ret < 0) {
                 if (tmp) {
-                    flb_free(tmp);
+                    flb_sds_destroy(tmp);
                 }
                 return -1;
             }
