@@ -24,6 +24,7 @@
 #include <fluent-bit/flb_env.h>
 #include <fluent-bit/flb_router.h>
 #include <fluent-bit/flb_mp.h>
+#include <fluent-bit/flb_kv.h>
 #include <fluent-bit/flb_pack.h>
 #include <chunkio/chunkio.h>
 
@@ -73,8 +74,6 @@ void flb_filter_do(struct flb_input_chunk *ic,
     ssize_t write_at;
     struct mk_list *head;
     struct flb_filter_instance *f_ins;
-    msgpack_zone *mp_zone = NULL;
-
 
     /* For the incoming Tag make sure to create a NULL terminated reference */
     ntag = flb_malloc(tag_len + 1);
@@ -90,9 +89,6 @@ void flb_filter_do(struct flb_input_chunk *ic,
     work_data = (const char *) data;
     work_size = bytes;
 
-    /* Count number of incoming records */
-    mp_zone = msgpack_zone_new(MSGPACK_ZONE_CHUNK_SIZE);
-
     /* Iterate filters */
     mk_list_foreach(head, &config->filters) {
         f_ins = mk_list_entry(head, struct flb_filter_instance, _head);
@@ -114,7 +110,7 @@ void flb_filter_do(struct flb_input_chunk *ic,
 
 #ifdef FLB_HAVE_METRICS
             /* Count number of incoming records */
-            in_records = flb_mp_count_zone(work_data, work_size, mp_zone);
+            in_records = flb_mp_count(work_data, work_size);
 #endif
 
             /* Invoke the filter callback */
@@ -138,14 +134,13 @@ void flb_filter_do(struct flb_input_chunk *ic,
                     /* Summarize all records removed */
                     flb_metrics_sum(FLB_METRIC_N_DROPPED,
                                     in_records, f_ins->metrics);
-                    msgpack_zone_clear(mp_zone);
 #endif
 
                     break;
                 }
                 else {
 #ifdef FLB_HAVE_METRICS
-                    out_records = flb_mp_count_zone(out_buf, out_size, mp_zone);
+                    out_records = flb_mp_count(out_buf, out_size);
                     if (out_records > in_records) {
                         diff = (out_records - in_records);
                         /* Summarize new records */
@@ -160,8 +155,6 @@ void flb_filter_do(struct flb_input_chunk *ic,
                     }
 #endif
                 }
-                msgpack_zone_clear(mp_zone);
-
                 ret = flb_input_chunk_write_at(ic, write_at,
                                                out_buf, out_size);
                 if (ret == -1) {
@@ -174,15 +167,18 @@ void flb_filter_do(struct flb_input_chunk *ic,
                 /* Point back the 'data' pointer to the new address */
                 ret = cio_chunk_get_content(ic->chunk,
                                             (char **) &work_data, &cur_size);
-
-                work_data += (cur_size - out_size);
-                work_size = out_size;
+                if (ret != CIO_OK) {
+                    flb_error("[filter] error retrieving data chunk");
+                }
+                else {
+                    work_data += (cur_size - out_size);
+                    work_size = out_size;
+                }
                 flb_free(out_buf);
             }
         }
     }
 
-    msgpack_zone_free(mp_zone);
     flb_free(ntag);
 }
 
@@ -190,8 +186,8 @@ int flb_filter_set_property(struct flb_filter_instance *filter,
                             const char *k, const char *v)
 {
     int len;
-    char *tmp;
-    struct flb_config_prop *prop;
+    flb_sds_t tmp;
+    struct flb_kv *kv;
 
     len = strlen(k);
     tmp = flb_env_var_translate(filter->config->env, v);
@@ -203,6 +199,7 @@ int flb_filter_set_property(struct flb_filter_instance *filter,
 #ifdef FLB_HAVE_REGEX
     if (prop_key_check("match_regex", k, len) == 0) {
         filter->match_regex = flb_regex_create(tmp);
+        flb_sds_destroy(tmp);
     }
     else
 #endif
@@ -213,24 +210,27 @@ int flb_filter_set_property(struct flb_filter_instance *filter,
         filter->alias = tmp;
     }
     else {
-        /* Append any remaining configuration key to prop list */
-        prop = flb_malloc(sizeof(struct flb_config_prop));
-        if (!prop) {
-            flb_free(tmp);
+        /*
+         * Create the property, we don't pass the value since we will
+         * map it directly to avoid an extra memory allocation.
+         */
+        kv = flb_kv_item_create(&filter->properties, (char *) k, NULL);
+        if (!kv) {
+            if (tmp) {
+                flb_sds_destroy(tmp);
+            }
             return -1;
         }
-
-        prop->key = flb_strdup(k);
-        prop->val = tmp;
-        mk_list_add(&prop->_head, &filter->properties);
+        kv->val = tmp;
     }
 
     return 0;
 }
 
-const char *flb_filter_get_property(const char *key, struct flb_filter_instance *i)
+const char *flb_filter_get_property(const char *key,
+                                    struct flb_filter_instance *i)
 {
-    return flb_config_prop_get(key, &i->properties);
+    return flb_kv_get_key_value(key, &i->properties);
 }
 
 /* Invoke exit call for the filter plugin */
@@ -238,9 +238,6 @@ void flb_filter_exit(struct flb_config *config)
 {
     struct mk_list *tmp;
     struct mk_list *head;
-    struct mk_list *tmp_prop;
-    struct mk_list *head_prop;
-    struct flb_config_prop *prop;
     struct flb_filter_instance *ins;
     struct flb_filter_plugin *p;
 
@@ -254,18 +251,10 @@ void flb_filter_exit(struct flb_config *config)
         }
 
         /* release properties */
-        mk_list_foreach_safe(head_prop, tmp_prop, &ins->properties) {
-            prop = mk_list_entry(head_prop, struct flb_config_prop, _head);
-
-            flb_free(prop->key);
-            flb_free(prop->val);
-
-            mk_list_del(&prop->_head);
-            flb_free(prop);
-        }
+        flb_kv_release(&ins->properties);
 
         if (ins->match != NULL) {
-            flb_free(ins->match);
+            flb_sds_destroy(ins->match);
         }
 
 #ifdef FLB_HAVE_REGEX
@@ -281,7 +270,7 @@ void flb_filter_exit(struct flb_config *config)
         }
 #endif
         if (ins->alias) {
-            flb_free(ins->alias);
+            flb_sds_destroy(ins->alias);
         }
 
         mk_list_del(&ins->_head);
@@ -360,9 +349,6 @@ void flb_filter_initialize_all(struct flb_config *config)
 #endif
     struct mk_list *tmp;
     struct mk_list *head;
-    struct mk_list *tmp_prop;
-    struct mk_list *head_prop;
-    struct flb_config_prop *prop;
     struct flb_filter_plugin *p;
     struct flb_filter_instance *in;
 
@@ -412,16 +398,10 @@ void flb_filter_initialize_all(struct flb_config *config)
                 flb_error("Failed initialize filter %s", in->name);
 
                 /* release properties */
-                mk_list_foreach_safe(head_prop, tmp_prop, &in->properties) {
-                    prop = mk_list_entry(head_prop, struct flb_config_prop, _head);
-                    flb_free(prop->key);
-                    flb_free(prop->val);
-                    mk_list_del(&prop->_head);
-                    flb_free(prop);
-                }
+                flb_kv_release(&in->properties);
 
                 if (in->match != NULL) {
-                    flb_free(in->match);
+                    flb_sds_destroy(in->match);
                 }
 
 #ifdef FLB_HAVE_REGEX
