@@ -33,6 +33,7 @@
 #include <fluent-bit/flb_engine.h>
 #include <fluent-bit/flb_metrics.h>
 #include <fluent-bit/flb_storage.h>
+#include <fluent-bit/flb_kv.h>
 
 #define protcmp(a, b)  strncasecmp(a, b, strlen(a))
 
@@ -116,7 +117,7 @@ struct flb_input_instance *flb_input_new(struct flb_config *config,
         }
 
         /* Create plugin instance */
-        instance = flb_malloc(sizeof(struct flb_input_instance));
+        instance = flb_calloc(1, sizeof(struct flb_input_instance));
         if (!instance) {
             flb_errno();
             return NULL;
@@ -152,10 +153,12 @@ struct flb_input_instance *flb_input_new(struct flb_config *config,
         mk_list_init(&instance->routes);
         mk_list_init(&instance->tasks);
         mk_list_init(&instance->chunks);
-        mk_list_init(&instance->properties);
         mk_list_init(&instance->collectors);
         mk_list_init(&instance->threads);
 
+        /* Initialize properties list */
+        flb_kv_init(&instance->properties);
+
         /* Plugin use networking */
         if (plugin->flags & FLB_INPUT_NET) {
             ret = flb_net_host_set(plugin->name, &instance->host, input);
@@ -200,14 +203,14 @@ int flb_input_set_property(struct flb_input_instance *in,
 {
     int len;
     ssize_t limit;
-    char *tmp;
-    struct flb_config_prop *prop;
+    flb_sds_t tmp = NULL;
+    struct flb_kv *kv;
 
     len = strlen(k);
     tmp = flb_env_var_translate(in->config->env, v);
     if (tmp) {
-        if (strlen(tmp) == 0) {
-            flb_free(tmp);
+        if (flb_sds_len(tmp) == 0) {
+            flb_sds_destroy(tmp);
             tmp = NULL;
         }
     }
@@ -215,18 +218,18 @@ int flb_input_set_property(struct flb_input_instance *in,
     /* Check if the key is a known/shared property */
     if (prop_key_check("tag", k, len) == 0 && tmp) {
         in->tag     = tmp;
-        in->tag_len = strlen(tmp);
+        in->tag_len = flb_sds_len(tmp);
     }
     else if (prop_key_check("routable", k, len) == 0 && tmp) {
         in->routable = flb_utils_bool(tmp);
-        flb_free(tmp);
+        flb_sds_destroy(tmp);
     }
     else if (prop_key_check("alias", k, len) == 0 && tmp) {
         in->alias = tmp;
     }
     else if (prop_key_check("mem_buf_limit", k, len) == 0 && tmp) {
         limit = flb_utils_size_to_bytes(tmp);
-        flb_free(tmp);
+        flb_sds_destroy(tmp);
         if (limit == -1) {
             return -1;
         }
@@ -241,26 +244,26 @@ int flb_input_set_property(struct flb_input_instance *in,
     else if (prop_key_check("port", k, len) == 0) {
         if (tmp) {
             in->host.port = atoi(tmp);
-            flb_free(tmp);
+            flb_sds_destroy(tmp);
         }
     }
     else if (prop_key_check("ipv6", k, len) == 0 && tmp) {
         in->host.ipv6 = flb_utils_bool(tmp);
-        flb_free(tmp);
+        flb_sds_destroy(tmp);
     }
     else {
-        /* Append any remaining configuration key to prop list */
-        prop = flb_malloc(sizeof(struct flb_config_prop));
-        if (!prop) {
+        /*
+         * Create the property, we don't pass the value since we will
+         * map it directly to avoid an extra memory allocation.
+         */
+        kv = flb_kv_item_create(&in->properties, (char *) k, NULL);
+        if (!kv) {
             if (tmp) {
-                flb_free(tmp);
+                flb_sds_destroy(tmp);
             }
             return -1;
         }
-
-        prop->key = flb_strdup(k);
-        prop->val = tmp;
-        mk_list_add(&prop->_head, &in->properties);
+        kv->val = tmp;
     }
 
     return 0;
@@ -283,37 +286,26 @@ const char *flb_input_name(struct flb_input_instance *in)
 
 void flb_input_instance_free(struct flb_input_instance *in)
 {
-    struct mk_list *head_prop;
-    struct flb_config_prop *prop;
-    struct mk_list *tmp_prop;
-
     if (in->alias) {
-        flb_free(in->alias);
+        flb_sds_destroy(in->alias);
     }
 
     /* Remove URI context */
     if (in->host.uri) {
         flb_uri_destroy(in->host.uri);
     }
-    flb_free(in->host.name);
-    flb_free(in->host.address);
+
+    flb_sds_destroy(in->host.name);
+    flb_sds_destroy(in->host.address);
 
     /* release the tag if any */
-    flb_free(in->tag);
+    flb_sds_destroy(in->tag);
 
     /* Let the engine remove any pending task */
     flb_engine_destroy_tasks(&in->tasks);
 
     /* release properties */
-    mk_list_foreach_safe(head_prop, tmp_prop, &in->properties) {
-        prop = mk_list_entry(head_prop, struct flb_config_prop, _head);
-
-        flb_free(prop->key);
-        flb_free(prop->val);
-
-        mk_list_del(&prop->_head);
-        flb_free(prop);
-    }
+    flb_kv_release(&in->properties);
 
     /* Remove metrics */
 #ifdef FLB_HAVE_METRICS
@@ -526,6 +518,11 @@ int flb_input_set_collector_time(struct flb_input_instance *in,
     struct flb_input_collector *collector;
 
     collector = flb_malloc(sizeof(struct flb_input_collector));
+    if (!collector) {
+        flb_errno();
+        return -1;
+    }
+
     collector->id          = collector_id(in);
     collector->type        = FLB_COLLECT_TIME;
     collector->cb_collect  = cb_collect;
@@ -551,6 +548,11 @@ int flb_input_set_collector_event(struct flb_input_instance *in,
     struct flb_input_collector *collector;
 
     collector = flb_malloc(sizeof(struct flb_input_collector));
+    if (!collector) {
+        flb_errno();
+        return -1;
+    }
+
     collector->id          = collector_id(in);
     collector->type        = FLB_COLLECT_FD_EVENT;
     collector->cb_collect  = cb_collect;
@@ -799,6 +801,11 @@ int flb_input_set_collector_socket(struct flb_input_instance *in,
     struct flb_input_collector *collector;
 
     collector = flb_malloc(sizeof(struct flb_input_collector));
+    if (!collector) {
+        flb_errno();
+        return -1;
+    }
+
     collector->type        = FLB_COLLECT_FD_SERVER;
     collector->cb_collect  = cb_new_connection;
     collector->fd_event    = fd;
