@@ -41,9 +41,6 @@
  *
  * Note that Upstreams context may define how network operations will work,
  * basically synchronous or asynchronous (non-blocking).
- *
- * When Fluent Bit is set with config->flush_mode = FLB_FLUSH_PTHREADS, all
- * operations will work in blocking (synchronous) mode.
  */
 
 #include <stdio.h>
@@ -194,7 +191,6 @@ FLB_INLINE int flb_io_net_connect(struct flb_upstream_conn *u_conn,
     }
 #endif
 
-    u_conn->connect_count++;
     flb_trace("[io] connection OK");
 
     return 0;
