@@ -28,7 +28,7 @@
 #include <fluent-bit/flb_env.h>
 #include <fluent-bit/flb_thread.h>
 #include <fluent-bit/flb_output.h>
-
+#include <fluent-bit/flb_kv.h>
 #include <fluent-bit/flb_io.h>
 #include <fluent-bit/flb_uri.h>
 #include <fluent-bit/flb_config.h>
@@ -80,35 +80,26 @@ void flb_output_pre_run(struct flb_config *config)
 
 static void flb_output_free_properties(struct flb_output_instance *ins)
 {
-    struct mk_list *tmp;
-    struct mk_list *head;
-    struct flb_config_prop *prop;
-
-    mk_list_foreach_safe(head, tmp, &ins->properties) {
-        prop = mk_list_entry(head, struct flb_config_prop, _head);
-
-        flb_free(prop->key);
-        flb_free(prop->val);
-
-        mk_list_del(&prop->_head);
-        flb_free(prop);
-    }
+    flb_kv_release(&ins->properties);
 
 #ifdef FLB_HAVE_TLS
+    if (ins->tls_vhost) {
+        flb_sds_destroy(ins->tls_vhost);
+    }
     if (ins->tls_ca_path) {
-        flb_free(ins->tls_ca_path);
+        flb_sds_destroy(ins->tls_ca_path);
     }
     if (ins->tls_ca_file) {
-        flb_free(ins->tls_ca_file);
+        flb_sds_destroy(ins->tls_ca_file);
     }
     if (ins->tls_crt_file) {
-        flb_free(ins->tls_crt_file);
+        flb_sds_destroy(ins->tls_crt_file);
     }
     if (ins->tls_key_file) {
-        flb_free(ins->tls_key_file);
+        flb_sds_destroy(ins->tls_key_file);
     }
     if (ins->tls_key_passwd) {
-        flb_free(ins->tls_key_passwd);
+        flb_sds_destroy(ins->tls_key_passwd);
     }
 #endif
 }
@@ -116,7 +107,7 @@ static void flb_output_free_properties(struct flb_output_instance *ins)
 int flb_output_instance_destroy(struct flb_output_instance *ins)
 {
     if (ins->alias) {
-        flb_free(ins->alias);
+        flb_sds_destroy(ins->alias);
     }
 
     /* Remove URI context */
@@ -124,9 +115,9 @@ int flb_output_instance_destroy(struct flb_output_instance *ins)
         flb_uri_destroy(ins->host.uri);
     }
 
-    flb_free(ins->host.name);
-    flb_free(ins->host.address);
-    flb_free(ins->match);
+    flb_sds_destroy(ins->host.name);
+    flb_sds_destroy(ins->host.address);
+    flb_sds_destroy(ins->match);
 
 #ifdef FLB_HAVE_REGEX
         if (ins->match_regex) {
@@ -135,7 +126,7 @@ int flb_output_instance_destroy(struct flb_output_instance *ins)
 #endif
 
 #ifdef FLB_HAVE_TLS
-    if (ins->flags & FLB_IO_TLS) {
+    if (ins->use_tls == FLB_TRUE) {
         if (ins->tls.context) {
             flb_tls_context_destroy(ins->tls.context);
         }
@@ -191,7 +182,7 @@ static inline int instance_id(struct flb_config *config)
         return 0;
     }
 
-    entry = mk_list_entry_last(&config->filters, struct flb_output_instance,
+    entry = mk_list_entry_last(&config->outputs, struct flb_output_instance,
                                _head);
     return (entry->id + 1);
 }
@@ -276,7 +267,8 @@ struct flb_output_instance *flb_output_new(struct flb_config *config,
 
         ctx = flb_calloc(1, sizeof(struct flb_plugin_proxy_context));
         if (!ctx) {
-            perror("calloc");
+            flb_errno();
+            flb_free(instance);
             return NULL;
         }
 
@@ -295,6 +287,7 @@ struct flb_output_instance *flb_output_new(struct flb_config *config,
 #endif
     instance->retry_limit = 1;
     instance->host.name   = NULL;
+    instance->host.address = NULL;
 
     /* Parent plugin flags */
     flags = instance->flags;
@@ -310,10 +303,15 @@ struct flb_output_instance *flb_output_new(struct flb_config *config,
         instance->flags |= FLB_IO_TLS;
     }
 
+    /* Keepalive */
+    instance->keepalive = FLB_FALSE;
+    instance->keepalive_timeout = FLB_OUTPUT_KA_TIMEOUT;
+
 #ifdef FLB_HAVE_TLS
     instance->tls.context    = NULL;
     instance->tls_debug      = -1;
     instance->tls_verify     = FLB_TRUE;
+    instance->tls_vhost      = NULL;
     instance->tls_ca_path    = NULL;
     instance->tls_ca_file    = NULL;
     instance->tls_crt_file   = NULL;
@@ -328,7 +326,8 @@ struct flb_output_instance *flb_output_new(struct flb_config *config,
             return NULL;
         }
     }
-    mk_list_init(&instance->properties);
+
+    flb_kv_init(&instance->properties);
     mk_list_add(&instance->_head, &config->outputs);
 
     return instance;
@@ -351,14 +350,14 @@ int flb_output_set_property(struct flb_output_instance *out,
                             const char *k, const char *v)
 {
     int len;
-    char *tmp;
-    struct flb_config_prop *prop;
+    flb_sds_t tmp;
+    struct flb_kv *kv;
 
     len = strlen(k);
     tmp = flb_env_var_translate(out->config->env, v);
     if (tmp) {
         if (strlen(tmp) == 0) {
-            flb_free(tmp);
+            flb_sds_destroy(tmp);
             tmp = NULL;
         }
     }
@@ -368,9 +367,9 @@ int flb_output_set_property(struct flb_output_instance *out,
         out->match = tmp;
     }
 #ifdef FLB_HAVE_REGEX
-    else if (prop_key_check("match_regex", k, len) == 0) {
+    else if (prop_key_check("match_regex", k, len) == 0 && tmp) {
         out->match_regex = flb_regex_create(tmp);
-        flb_free(tmp);
+        flb_sds_destroy(tmp);
     }
 #endif
     else if (prop_key_check("alias", k, len) == 0 && tmp) {
@@ -382,15 +381,33 @@ int flb_output_set_property(struct flb_output_instance *out,
     else if (prop_key_check("port", k, len) == 0) {
         if (tmp) {
             out->host.port = atoi(tmp);
-            flb_free(tmp);
+            flb_sds_destroy(tmp);
         }
         else {
             out->host.port = 0;
         }
     }
+    else if (prop_key_check("keepalive", k, len) == 0){
+        if (tmp) {
+            out->keepalive = flb_utils_bool(tmp);
+            flb_sds_destroy(tmp);
+        }
+        else {
+            out->keepalive = FLB_FALSE;
+        }
+    }
+    else if (prop_key_check("keepalive_timeout", k, len) == 0){
+        if (tmp) {
+            out->keepalive_timeout = atoi(tmp);
+            flb_sds_destroy(tmp);
+        }
+        else {
+            out->keepalive_timeout = 10;
+        }
+    }
     else if (prop_key_check("ipv6", k, len) == 0 && tmp) {
         out->host.ipv6 = flb_utils_bool(tmp);
-        flb_free(tmp);
+        flb_sds_destroy(tmp);
     }
     else if (prop_key_check("retry_limit", k, len) == 0) {
         if (tmp) {
@@ -402,7 +419,7 @@ int flb_output_set_property(struct flb_output_instance *out,
             else {
                 out->retry_limit = atoi(tmp);
             }
-            flb_free(tmp);
+            flb_sds_destroy(tmp);
         }
         else {
             out->retry_limit = 0;
@@ -413,7 +430,7 @@ int flb_output_set_property(struct flb_output_instance *out,
         if (strcasecmp(tmp, "true") == 0 || strcasecmp(tmp, "on") == 0) {
             if ((out->flags & FLB_IO_TLS) == 0) {
                 flb_error("[config] %s don't support TLS", out->name);
-                flb_free(tmp);
+                flb_sds_destroy(tmp);
                 return -1;
             }
 
@@ -422,7 +439,7 @@ int flb_output_set_property(struct flb_output_instance *out,
         else {
             out->use_tls = FLB_FALSE;
         }
-        flb_free(tmp);
+        flb_sds_destroy(tmp);
     }
     else if (prop_key_check("tls.verify", k, len) == 0 && tmp) {
         if (strcasecmp(tmp, "true") == 0 || strcasecmp(tmp, "on") == 0) {
@@ -431,11 +448,14 @@ int flb_output_set_property(struct flb_output_instance *out,
         else {
             out->tls_verify = FLB_FALSE;
         }
-        flb_free(tmp);
+        flb_sds_destroy(tmp);
     }
     else if (prop_key_check("tls.debug", k, len) == 0 && tmp) {
         out->tls_debug = atoi(tmp);
-        flb_free(tmp);
+        flb_sds_destroy(tmp);
+    }
+    else if (prop_key_check("tls.vhost", k, len) == 0) {
+        out->tls_vhost = tmp;
     }
     else if (prop_key_check("tls.ca_path", k, len) == 0) {
         out->tls_ca_path = tmp;
@@ -454,18 +474,18 @@ int flb_output_set_property(struct flb_output_instance *out,
     }
 #endif
     else {
-        /* Append any remaining configuration key to prop list */
-        prop = flb_malloc(sizeof(struct flb_config_prop));
-        if (!prop) {
+        /*
+         * Create the property, we don't pass the value since we will
+         * map it directly to avoid an extra memory allocation.
+         */
+        kv = flb_kv_item_create(&out->properties, (char *) k, NULL);
+        if (!kv) {
             if (tmp) {
-                flb_free(tmp);
+                flb_sds_destroy(tmp);
             }
             return -1;
         }
-
-        prop->key = flb_strdup(k);
-        prop->val = tmp;
-        mk_list_add(&prop->_head, &out->properties);
+        kv->val = tmp;
     }
 
     return 0;
@@ -477,7 +497,7 @@ void flb_output_net_default(const char *host, const int port,
 {
     /* Set default network configuration */
     if (!o_ins->host.name) {
-        o_ins->host.name = flb_strdup(host);
+        o_ins->host.name = flb_sds_create(host);
     }
     if (o_ins->host.port == 0) {
         o_ins->host.port = port;
@@ -551,9 +571,10 @@ int flb_output_init(struct flb_config *config)
 #endif
 
 #ifdef FLB_HAVE_TLS
-        if (ins->flags & FLB_IO_TLS) {
+        if (ins->use_tls == FLB_TRUE) {
             ins->tls.context = flb_tls_context_new(ins->tls_verify,
                                                    ins->tls_debug,
+                                                   ins->tls_vhost,
                                                    ins->tls_ca_path,
                                                    ins->tls_ca_file,
                                                    ins->tls_crt_file,
@@ -593,3 +614,47 @@ int flb_output_check(struct flb_config *config)
     }
     return 0;
 }
+
+/*
+ * Output plugins might have enabled certain features that have not been passed
+ * directly to the upstream context. In order to avoid let plugins validate specific
+ * variables from the instance context like tls, tls.x, keepalive, etc, we populate
+ * them directly through this function.
+ */
+int flb_output_upstream_set(struct flb_upstream *u, struct flb_output_instance *ins)
+{
+    int flags = 0;
+
+    if (!u) {
+        return -1;
+    }
+
+    /* TLS */
+#ifdef FLB_HAVE_TLS
+    if (ins->use_tls == FLB_TRUE) {
+        flags |= FLB_IO_TLS;
+    }
+    else {
+        flags |= FLB_IO_TCP;
+    }
+#else
+    flags |= FLB_IO_TCP;
+#endif
+
+    /* IPv6 */
+    if (ins->host.ipv6 == FLB_TRUE) {
+        flags |= FLB_IO_IPV6;
+    }
+
+    /* KeepAlive */
+    if (ins->keepalive == FLB_TRUE) {
+        flags |= FLB_IO_TCP_KA;
+
+        /* Keepalive timeout */
+        u->ka_timeout = ins->keepalive_timeout;
+    }
+
+    /* Set flags */
+    u->flags |= flags;
+    return 0;
+}
