@@ -21,11 +21,15 @@
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_mem.h>
 #include <fluent-bit/flb_str.h>
+#include <fluent-bit/flb_sds.h>
 #include <fluent-bit/flb_input.h>
 #include <fluent-bit/flb_output.h>
 #include <fluent-bit/flb_config.h>
 #include <fluent-bit/flb_router.h>
+
+#ifdef FLB_HAVE_REGEX
 #include <onigmo.h>
+#endif
 
 #include <string.h>
 
@@ -157,7 +161,7 @@ int flb_router_io_set(struct flb_config *config)
             ) {
             flb_debug("[router] default match rule %s:%s",
                       i_ins->name, o_ins->name);
-            o_ins->match = flb_strdup("*");
+            o_ins->match = flb_sds_create_len("*", 1);
             flb_router_connect(i_ins, o_ins);
             return 0;
         }
