@@ -65,6 +65,7 @@ static inline void map_free_task_id(int id, struct flb_config *config)
     config->tasks_map[id].task = NULL;
 }
 
+
 void flb_task_retry_destroy(struct flb_task_retry *retry)
 {
     int ret;
@@ -80,6 +81,38 @@ void flb_task_retry_destroy(struct flb_task_retry *retry)
     flb_free(retry);
 }
 
+/*
+ * For an existing task 'retry', re-schedule it. One of the use case of this function
+ * is when the engine dispatcher fails to bring the chunk up due to Chunk I/O
+ * configuration restrictions, the task needs to be re-scheduled.
+ */
+int flb_task_retry_reschedule(struct flb_task_retry *retry, struct flb_config *config)
+{
+    int seconds;
+    struct flb_task *task;
+
+    task = retry->parent;
+    seconds = flb_sched_request_create(config, retry, retry->attemps);
+    seconds = -1;
+    if (seconds == -1) {
+        /*
+         * This is the worse case scenario: 'cannot re-schedule a retry'. If the Chunk
+         * resides only in memory, it will be lost.  */
+        flb_warn("[task] retry for task %i could not be re-scheduled", task->id);
+        flb_task_retry_destroy(retry);
+        if (task->users == 0 && mk_list_size(&task->retries) == 0) {
+            flb_task_destroy(task, FLB_TRUE);
+        }
+        return -1;
+    }
+    else {
+        flb_info("[task] re-schedule retry=%p %i in the next %i seconds",
+                  retry, task->id, seconds);
+    }
+
+    return 0;
+}
+
 struct flb_task_retry *flb_task_retry_create(struct flb_task *task,
                                              void *data)
 {
@@ -96,7 +129,7 @@ struct flb_task_retry *flb_task_retry_create(struct flb_task *task,
     mk_list_foreach_safe(head, tmp, &task->retries) {
         retry = mk_list_entry(head, struct flb_task_retry, _head);
         if (retry->o_ins == o_ins) {
-            if (retry->attemps > o_ins->retry_limit && o_ins->retry_limit >= 0) {
+            if (retry->attemps >= o_ins->retry_limit && o_ins->retry_limit >= 0) {
                 flb_debug("[task] task_id=%i reached retry-attemps limit %i/%i",
                           task->id, retry->attemps, o_ins->retry_limit);
                 flb_task_retry_destroy(retry);
@@ -140,6 +173,40 @@ struct flb_task_retry *flb_task_retry_create(struct flb_task *task,
     return retry;
 }
 
+/*
+ * Return FLB_TRUE or FLB_FALSE if the chunk pointed by the task was
+ * created on this running instance or it comes from a chunk in the
+ * filesystem from a previous run.
+ */
+int flb_task_from_fs_storage(struct flb_task *task)
+{
+    struct flb_input_chunk *ic;
+
+    ic = (struct flb_input_chunk *) task->ic;
+    return ic->fs_backlog;
+}
+
+int flb_task_retry_count(struct flb_task *task, void *data)
+{
+    struct mk_list *head;
+    struct flb_task_retry *retry;
+    struct flb_output_instance *o_ins;
+    struct flb_output_thread *out_th;
+
+    out_th = (struct flb_output_thread *) FLB_THREAD_DATA(data);
+    o_ins = out_th->o_ins;
+
+    /* Delete 'retries' only associated with the output instance */
+    mk_list_foreach(head, &task->retries) {
+        retry = mk_list_entry(head, struct flb_task_retry, _head);
+        if (retry->o_ins == o_ins) {
+            return retry->attemps;
+        }
+    }
+
+    return -1;
+}
+
 /* Check if a 'retry' context exists for a specific task, if so, cleanup */
 int flb_task_retry_clean(struct flb_task *task, void *data)
 {
@@ -189,7 +256,6 @@ static struct flb_task *task_alloc(struct flb_config *config)
 
     /* Initialize minimum variables */
     task->id        = task_id;
-    task->mapped    = FLB_FALSE;
     task->config    = config;
     task->status    = FLB_TASK_NEW;
     task->n_threads = 0;
@@ -246,7 +312,6 @@ struct flb_task *flb_task_create(uint64_t ref_id,
     task->size   = size;
     task->i_ins  = i_ins;
     task->ic     = ic;
-    task->destinations = 0;
     mk_list_add(&task->_head, &i_ins->tasks);
 
     /* Find matching routes for the incoming tag */
