@@ -123,37 +123,11 @@ struct flb_upstream *flb_upstream_create_url(struct flb_config *config,
     return u;
 }
 
-int flb_upstream_destroy(struct flb_upstream *u)
-{
-    struct mk_list *tmp;
-    struct mk_list *head;
-    struct flb_upstream_conn *u_conn;
-
-    mk_list_foreach_safe(head, tmp, &u->av_queue) {
-        u_conn = mk_list_entry(head, struct flb_upstream_conn, _head);
-        flb_upstream_conn_release(u_conn);
-    }
-
-    mk_list_foreach_safe(head, tmp, &u->busy_queue) {
-        u_conn = mk_list_entry(head, struct flb_upstream_conn, _head);
-        flb_upstream_conn_release(u_conn);
-    }
-
-    flb_free(u->tcp_host);
-    flb_free(u);
-
-    return 0;
-}
-
 static struct flb_upstream_conn *create_conn(struct flb_upstream *u)
 {
     int ret;
     struct flb_upstream_conn *conn;
-#if defined (FLB_HAVE_FLUSH_LIBCO)
     struct flb_thread *th = pthread_getspecific(flb_thread_key);
-#else
-    void *th = NULL;
-#endif
 
     conn = flb_malloc(sizeof(struct flb_upstream_conn));
     if (!conn) {
@@ -162,10 +136,12 @@ static struct flb_upstream_conn *create_conn(struct flb_upstream *u)
     }
     conn->u             = u;
     conn->fd            = -1;
-    conn->connect_count = 0;
 #ifdef FLB_HAVE_TLS
     conn->tls_session   = NULL;
 #endif
+    conn->ts_created = time(NULL);
+    conn->ts_available = 0;
+    conn->ka_count = 0;
 
     MK_EVENT_ZERO(&conn->event);
 
@@ -180,79 +156,25 @@ static struct flb_upstream_conn *create_conn(struct flb_upstream *u)
     mk_list_add(&conn->_head, &u->busy_queue);
     u->n_connections++;
 
-    return conn;
-}
-
-static struct flb_upstream_conn *get_conn(struct flb_upstream *u)
-{
-    struct flb_upstream_conn *conn;
-
-#ifdef FLB_HAVE_FLUSH_PTHREADS
-    pthread_mutex_lock(&u->mutex_queue);
-#endif
-    /* Get the first available connection and increase the counter */
-    conn = mk_list_entry_first(&u->av_queue,
-                               struct flb_upstream_conn, _head);
-    u->n_connections++;
-
-    /* Move it to the busy queue */
-    mk_list_del(&conn->_head);
-    mk_list_add(&conn->_head, &u->busy_queue);
-
-#ifdef FLB_HAVE_FLUSH_PTHREADS
-    pthread_mutex_unlock(&u->mutex_queue);
-#endif
+    if (conn->u->flags & FLB_IO_TCP_KA) {
+        flb_debug("[upstream] KA connection #%i to %s:%i is connected",
+                  conn->fd, u->tcp_host, u->tcp_port);
+    }
 
     return conn;
 }
 
-struct flb_upstream_conn *flb_upstream_conn_get(struct flb_upstream *u)
-{
-    struct flb_upstream_conn *u_conn = NULL;
-
-    /*
-     * FIXME: for 0.9 series the keep alive mode will be enabled, useless
-     * check now as the available queue is always empty.
-     */
-    if (mk_list_is_empty(&u->av_queue) == 0) {
-
-        if (u->max_connections <= 0) {
-            u_conn = create_conn(u);
-        }
-        else if (u->n_connections < u->max_connections) {
-            u_conn = create_conn(u);
-        }
-        else {
-            return NULL;
-        }
-    }
-    else {
-        /* Get an available connection */
-        u_conn = get_conn(u);
-    }
-
-    if (!u_conn) {
-        return NULL;
-    }
-
-    return u_conn;
-}
-
-int flb_upstream_conn_release(struct flb_upstream_conn *u_conn)
+static int destroy_conn(struct flb_upstream_conn *u_conn)
 {
     struct flb_upstream *u = u_conn->u;
 
-    flb_trace("[upstream] [fd=%i] releasing connection %p",
-              u_conn->fd, u_conn);
+    flb_trace("[upstream] destroy connection #%i to %s:%i",
+              u_conn->fd, u->tcp_host, u->tcp_port);
 
     if (u->flags & FLB_IO_ASYNC) {
         mk_event_del(u->evl, &u_conn->event);
     }
 
-    if (u_conn->fd > 0) {
-        flb_socket_close(u_conn->fd);
-    }
-
 #ifdef FLB_HAVE_TLS
     if (u_conn->tls_session) {
         flb_tls_session_destroy(u_conn->tls_session);
@@ -260,6 +182,10 @@ int flb_upstream_conn_release(struct flb_upstream_conn *u_conn)
     }
 #endif
 
+    if (u_conn->fd > 0) {
+        flb_socket_close(u_conn->fd);
+    }
+
     /* remove connection from the queue */
     mk_list_del(&u_conn->_head);
 
@@ -268,3 +194,155 @@ int flb_upstream_conn_release(struct flb_upstream_conn *u_conn)
 
     return 0;
 }
+
+int flb_upstream_destroy(struct flb_upstream *u)
+{
+    struct mk_list *tmp;
+    struct mk_list *head;
+    struct flb_upstream_conn *u_conn;
+
+    mk_list_foreach_safe(head, tmp, &u->av_queue) {
+        u_conn = mk_list_entry(head, struct flb_upstream_conn, _head);
+        destroy_conn(u_conn);
+    }
+
+    mk_list_foreach_safe(head, tmp, &u->busy_queue) {
+        u_conn = mk_list_entry(head, struct flb_upstream_conn, _head);
+        destroy_conn(u_conn);
+    }
+
+    flb_free(u->tcp_host);
+    flb_free(u);
+
+    return 0;
+}
+
+struct flb_upstream_conn *flb_upstream_conn_get(struct flb_upstream *u)
+{
+    time_t ts;
+    struct mk_list *tmp;
+    struct mk_list *head;
+    struct flb_upstream_conn *conn = NULL;
+
+    /* On non Keepalive mode, always create a new TCP connection */
+    if ((u->flags & FLB_IO_TCP_KA) == 0) {
+        return create_conn(u);
+    }
+
+    /*
+     * If we are in keepalive mode, iterate list of available connections,
+     * take a little of time to do some cleanup and assign a connection. If no
+     * entries exists, just create a new one.
+     */
+
+    ts = time(NULL);
+    mk_list_foreach_safe(head, tmp, &u->av_queue) {
+        conn = mk_list_entry(head, struct flb_upstream_conn, _head);
+
+        /* Check if is time to destroy this connection */
+        if ((ts - conn->ts_created) > u->ka_timeout) {
+            flb_debug("[upstream] KA connection #%i to %s:%i timed out, closing.",
+                      conn->fd, u->tcp_host, u->tcp_port);
+            destroy_conn(conn);
+            conn = NULL;
+            continue;
+        }
+
+        /* This connection works, let's move it to the busy queue */
+        mk_list_del(&conn->_head);
+        mk_list_add(&conn->_head, &u->busy_queue);
+        flb_debug("[upstream] KA connection #%i to %s:%i has been assigned (recycled)",
+                  conn->fd, u->tcp_host, u->tcp_port);
+
+        /*
+         * Note: since we are in a keepalive connection, the socket is already being
+         * monitored for possible disconnections while idle. Upon re-use by the caller
+         * when it try to send some data, the I/O interface (flb_io.c) will put the
+         * proper event mask and reuse, there is no need to remove the socket from
+         * the event loop and re-add it again.
+         *
+         * So... just return the connection context.
+         */
+        return conn;
+    }
+
+    /* No keepalive connection available, create a new one */
+    if (!conn) {
+        return create_conn(u);
+    }
+
+    return conn;
+}
+
+/*
+ * An 'idle' and keepalive might be disconnected, if so, this callback will perform
+ * the proper connection cleanup.
+ */
+static int cb_upstream_conn_ka_dropped(void *data)
+{
+    struct flb_upstream_conn *conn;
+
+    conn = (struct flb_upstream_conn *) data;
+
+    flb_debug("[upstream] KA connection #%i to %s:%i has been disconnected "
+              "by the remote service",
+              conn->fd, conn->u->tcp_host, conn->u->tcp_port);
+    return destroy_conn(conn);
+}
+
+int flb_upstream_conn_release(struct flb_upstream_conn *conn)
+{
+    int ret;
+    time_t ts;
+    struct flb_upstream *u;
+
+    /* Upstream context */
+    u = conn->u;
+
+    /* If this is a valid KA connection just recycle */
+    if (conn->u->flags & FLB_IO_TCP_KA) {
+        /* check keepalive timeout */
+        ts = time(NULL);
+        if ((ts - conn->ts_created) > conn->u->ka_timeout) {
+            /* this connection must be dropped */
+            flb_debug("[upstream] KA connection #%i to %s:%i timed out, closing.",
+                      conn->fd, conn->u->tcp_host, conn->u->tcp_port);
+            return destroy_conn(conn);
+        }
+
+        /*
+         * This connection is still useful, move it to the 'available' queue and
+         * initialize variables.
+         */
+        mk_list_del(&conn->_head);
+        mk_list_add(&conn->_head, &conn->u->av_queue);
+        conn->ts_available = time(NULL);
+
+        /*
+         * The socket at this point is not longer monitored, so if we want to be
+         * notified if the 'available keepalive connection' gets disconnected by
+         * the remote endpoint we need to add it again.
+         */
+        conn->event.handler = cb_upstream_conn_ka_dropped;
+        conn->event.data    = &conn;
+
+        ret = mk_event_add(u->evl, conn->fd,
+                           FLB_ENGINE_EV_CUSTOM,
+                           MK_EVENT_CLOSE, &conn->event);
+        if (ret == -1) {
+            /* We failed the registration, for safety just destroy the connection */
+            flb_debug("[upstream] KA connection #%i to %s:%i could not be "
+                      "registered, closing.",
+                      conn->fd, conn->u->tcp_host, conn->u->tcp_port);
+            return destroy_conn(conn);
+        }
+
+        flb_debug("[upstream] KA connection #%i to %s:%i is now available",
+                  conn->fd, conn->u->tcp_host, conn->u->tcp_port);
+        conn->ka_count++;
+        return 0;
+    }
+
+    /* No keepalive connections must be destroyed */
+    return destroy_conn(conn);
+}
