@@ -136,6 +136,37 @@ static void cb_mq_metrics(mk_mq_t *queue, void *data, size_t size)
     cleanup_metrics();
 }
 
+int string_cmp(const void* a_arg, const void* b_arg) {
+  char* a = *(char **)a_arg;
+  char* b = *(char **)b_arg;
+  return strcmp(a, b);
+}
+
+size_t extract_metric_name_end_position(char *s) {
+    int i;
+    for (i = 0; i < flb_sds_len(s); i++) {
+        if (s[i] == '{') {
+          return i;
+        }
+    }
+    return 0;
+}
+
+int is_same_metric(char *s1, char *s2) {
+  int i;
+  int p1 = extract_metric_name_end_position(s1);
+  int p2 = extract_metric_name_end_position(s2);
+  if (p1 != p2) {
+    return 0;
+  }
+  for (i = 0; i < p1; i++) {
+    if (s1[i] != s2[i]) {
+      return 0;
+    }
+  }
+  return 1;
+}
+
 /* API: expose metrics in Prometheus format /api/v1/metrics/prometheus */
 void cb_metrics_prometheus(mk_request_t *request, void *data)
 {
@@ -144,15 +175,23 @@ void cb_metrics_prometheus(mk_request_t *request, void *data)
     int m;
     int len;
     int time_len;
+    int start_time_len;
+    size_t index;
+    size_t num_metrics = 0;
     long now;
     flb_sds_t sds;
+    flb_sds_t sds_metric;
     size_t off = 0;
     struct flb_hs_buf *buf;
     msgpack_unpacked result;
     msgpack_object map;
     char tmp[32];
     char time_str[64];
+    char start_time_str[64];
+    char* *metrics_arr;
     struct timeval tp;
+    struct flb_hs *hs = data;
+    struct flb_config *config = hs->config;
 
     buf = metrics_get_latest();
     if (!buf) {
@@ -177,14 +216,30 @@ void cb_metrics_prometheus(mk_request_t *request, void *data)
     gettimeofday(&tp, NULL);
     now = tp.tv_sec * 1000 + tp.tv_usec / 1000;
     time_len = snprintf(time_str, sizeof(time_str) - 1, "%lu", now);
+    start_time_len = snprintf(start_time_str, sizeof(start_time_str) - 1, "%lu", config->init_time);
 
     /*
      * fluentbit_input_records[name="cpu0", hostname="${HOSTNAME}"] NUM TIMESTAMP
      * fluentbit_input_bytes[name="cpu0", hostname="${HOSTNAME}"] NUM TIMESTAMP
      */
+    index = 0;
     msgpack_unpacked_init(&result);
     msgpack_unpack_next(&result, buf->raw_data, buf->raw_size, &off);
     map = result.data;
+
+    /* we need to know number of exposed metrics to reserve a memory */
+    for (i = 0; i < map.via.map.size; i++) {
+        msgpack_object v = map.via.map.ptr[i].val;
+        /* Iterate sub-map */
+        for (j = 0; j < v.via.map.size; j++) {
+            msgpack_object sv = v.via.map.ptr[j].val;
+            for (m = 0; m < sv.via.map.size; m++) {
+                num_metrics++;
+            }
+        }
+    }
+    metrics_arr = flb_malloc(num_metrics * sizeof(char*));
+
     for (i = 0; i < map.via.map.size; i++) {
         msgpack_object k;
         msgpack_object v;
@@ -209,22 +264,59 @@ void cb_metrics_prometheus(mk_request_t *request, void *data)
                 mk = sv.via.map.ptr[m].key;
                 mv = sv.via.map.ptr[m].val;
 
-                sds = flb_sds_cat(sds, "fluentbit_", 10);
-                sds = flb_sds_cat(sds, k.via.str.ptr, k.via.str.size);
-                sds = flb_sds_cat(sds, "_", 1);
-                sds = flb_sds_cat(sds, mk.via.str.ptr, mk.via.str.size);
-                sds = flb_sds_cat(sds, "_total{name=\"", 13);
-                sds = flb_sds_cat(sds, sk.via.str.ptr, sk.via.str.size);
-                sds = flb_sds_cat(sds, "\"} ", 3);
+                /* Convert metric value to string */
+                len = snprintf(tmp, sizeof(tmp) - 1, "%" PRIu64 " ", mv.via.u64);
+                if (len < 0) {
+                    goto error;
+                }
 
-                len = snprintf(tmp, sizeof(tmp) - 1,
-                               "%" PRIu64 " ", mv.via.u64);
-                sds = flb_sds_cat(sds, tmp, len);
-                sds = flb_sds_cat(sds, time_str, time_len);
-                sds = flb_sds_cat(sds, "\n", 1);
+                /* Allocate buffer */
+                sds_metric = flb_sds_create_size(k.via.str.size
+                                                 + mk.via.str.size
+                                                 + sk.via.str.size
+                                                 + len + time_len + 28);
+                if (sds_metric == NULL) {
+                    goto error;
+                }
+
+                sds_metric = flb_sds_cat(sds_metric, "fluentbit_", 10);
+                sds_metric = flb_sds_cat(sds_metric, k.via.str.ptr, k.via.str.size);
+                sds_metric = flb_sds_cat(sds_metric, "_", 1);
+                sds_metric = flb_sds_cat(sds_metric, mk.via.str.ptr, mk.via.str.size);
+                sds_metric = flb_sds_cat(sds_metric, "_total{name=\"", 13);
+                sds_metric = flb_sds_cat(sds_metric, sk.via.str.ptr, sk.via.str.size);
+                sds_metric = flb_sds_cat(sds_metric, "\"} ", 3);
+                sds_metric = flb_sds_cat(sds_metric, tmp, len);
+                sds_metric = flb_sds_cat(sds_metric, time_str, time_len);
+                sds_metric = flb_sds_cat(sds_metric, "\n", 1);
+                metrics_arr[index] = sds_metric;
+                index++;
             }
         }
     }
+
+    /*  Sort metrics in alphabetic order, so we can group them later. */
+    qsort(metrics_arr, num_metrics, sizeof(char *), string_cmp);
+
+    /* When a new metric starts add TYPE annotation. */
+    sds = flb_sds_cat(sds, "# TYPE ", 7);
+    sds = flb_sds_cat(sds, metrics_arr[0], extract_metric_name_end_position(metrics_arr[0]));
+    sds = flb_sds_cat(sds, " counter\n", 9);
+    for (i = 0; i < num_metrics; i++) {
+        sds = flb_sds_cat(sds, metrics_arr[i], strlen(metrics_arr[i]));
+        if ((i != num_metrics - 1) && (is_same_metric(metrics_arr[i], metrics_arr[i+1]) == 0)) {
+            sds = flb_sds_cat(sds, "# TYPE ", 7);
+            sds = flb_sds_cat(sds, metrics_arr[i+1], extract_metric_name_end_position(metrics_arr[i+1]));
+            sds = flb_sds_cat(sds, " counter\n", 9);
+        }
+    }
+
+    /* Attach process_start_time_seconds metric. */
+    sds = flb_sds_cat(sds, "# TYPE process_start_time_seconds gauge\n", 40);
+    sds = flb_sds_cat(sds, "process_start_time_seconds ", 27);
+    sds = flb_sds_cat(sds, start_time_str, start_time_len);
+    sds = flb_sds_cat(sds, "\n", 1);
+
     msgpack_unpacked_destroy(&result);
     buf->users--;
 
@@ -233,8 +325,26 @@ void cb_metrics_prometheus(mk_request_t *request, void *data)
                    "Content-Type", 12,
                    PROMETHEUS_HEADER, sizeof(PROMETHEUS_HEADER) - 1);
     mk_http_send(request, sds, flb_sds_len(sds), NULL);
-    mk_http_done(request);
+    for (i = 0; i < num_metrics; i++) {
+      flb_sds_destroy(metrics_arr[i]);
+    }
+    flb_free(metrics_arr);
     flb_sds_destroy(sds);
+
+    mk_http_done(request);
+    return;
+
+error:
+    mk_http_status(request, 500);
+    mk_http_done(request);
+    buf->users--;
+
+    for (i = 0; i < index; i++) {
+      flb_sds_destroy(metrics_arr[i]);
+    }
+    flb_free(metrics_arr);
+    flb_sds_destroy(sds);
+    msgpack_unpacked_destroy(&result);
 }
 
 /* API: expose built-in metrics /api/v1/metrics */
