@@ -29,11 +29,29 @@
 #include <fluent-bit/flb_sds.h>
 #include <fluent-bit/flb_slist.h>
 #include <fluent-bit/stream_processor/flb_sp_parser.h>
+#include <fluent-bit/stream_processor/flb_sp_timeseries.h>
 #include <fluent-bit/stream_processor/flb_sp_record_func.h>
 
 #include "sql_parser.h"
 #include "sql_lex.h"
 
+static int swap_tmp_subkeys(struct mk_list **target, struct flb_sp_cmd *cmd)
+{
+    /* Map context keys into this command key structure */
+    *target = cmd->tmp_subkeys;
+
+    cmd->tmp_subkeys = flb_malloc(sizeof(struct mk_list));
+    if (!cmd->tmp_subkeys) {
+        flb_errno();
+        cmd->tmp_subkeys = *target;
+        cmd->status = FLB_SP_ERROR;
+        return -1;
+    }
+
+    flb_slist_create(cmd->tmp_subkeys);
+    return 0;
+}
+
 void flb_sp_cmd_destroy(struct flb_sp_cmd *cmd)
 {
     struct mk_list *head;
@@ -76,9 +94,35 @@ void flb_sp_cmd_destroy(struct flb_sp_cmd *cmd)
         flb_free(cmd->tmp_subkeys);
     }
 
+    if (cmd->tmp_params) {
+        flb_cmd_params_del(cmd->tmp_params);
+        flb_free(cmd->tmp_params);
+    }
+
     flb_free(cmd);
 }
 
+void flb_cmd_params_del(struct mk_list *params)
+{
+    struct flb_exp_param *p;
+    struct mk_list *head;
+    struct mk_list *tmp;
+
+    mk_list_foreach_safe(head, tmp, params) {
+        p = mk_list_entry(head, struct flb_exp_param, _head);
+        switch (p->type) {
+        case FLB_EXP_KEY:
+            flb_sp_cmd_key_del((struct flb_sp_cmd_key *) p->param);
+            break;
+        case FLB_EXP_STRING:
+            flb_sds_destroy(((struct flb_sp_value *) p->param)->val.string);
+            flb_free(p->param);
+            break;
+        }
+        flb_free(p);
+    }
+}
+
 void flb_sp_cmd_key_del(struct flb_sp_cmd_key *key)
 {
     if (key->name) {
@@ -94,6 +138,10 @@ void flb_sp_cmd_key_del(struct flb_sp_cmd_key *key)
         flb_slist_destroy(key->subkeys);
         flb_free(key->subkeys);
     }
+    if (key->timeseries_func) {
+        flb_cmd_params_del(&key->timeseries->params);
+        flb_free(key->timeseries);
+    }
     flb_free(key);
 }
 
@@ -102,16 +150,23 @@ void flb_sp_cmd_gb_key_del(struct flb_sp_cmd_gb_key *key)
     if (key->name) {
         flb_sds_destroy(key->name);
     }
+    if (key->subkeys) {
+        flb_slist_destroy(key->subkeys);
+        flb_free(key->subkeys);
+    }
     flb_free(key);
 }
 
-int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
-                       const char *key_name, const char *key_alias)
+struct flb_sp_cmd_key *flb_sp_key_create(struct flb_sp_cmd *cmd, int func,
+                                         const char *key_name,
+                                         const char *key_alias)
 {
     int s;
+    int ret;
     int aggr_func = 0;
     int time_func = 0;
     int record_func = 0;
+    int timeseries_func = 0;
     char *tmp;
     struct mk_list *head;
     struct flb_sp_cmd_key *key;
@@ -129,13 +184,18 @@ int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
         /* Record function */
         record_func = func;
     }
+    else if (func >= FLB_SP_TIMESERIES_START && func <= FLB_SP_TIMESERIES_END) {
+        /* Timeseries function */
+        timeseries_func = func;
+    }
 
     key = flb_calloc(1, sizeof(struct flb_sp_cmd_key));
     if (!key) {
         flb_errno();
         cmd->status = FLB_SP_ERROR;
-        return -1;
+        return NULL;
     }
+    key->gb_key = NULL;
     key->subkeys = NULL;
 
     /* key name and aliases works when the selection is not a wildcard */
@@ -144,7 +204,7 @@ int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
         if (!key->name) {
             flb_sp_cmd_key_del(key);
             cmd->status = FLB_SP_ERROR;
-            return -1;
+            return NULL;
         }
     }
     else {
@@ -154,10 +214,10 @@ int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
          * - aggregation using COUNT(*)
          */
         if (mk_list_size(&cmd->keys) > 0 && aggr_func == 0 &&
-            record_func == 0 && time_func == 0) {
+            record_func == 0 && time_func == 0 && timeseries_func == 0) {
             flb_sp_cmd_key_del(key);
             cmd->status = FLB_SP_ERROR;
-            return -1;
+            return NULL;
         }
     }
 
@@ -166,7 +226,7 @@ int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
         if (!key->alias) {
             flb_sp_cmd_key_del(key);
             cmd->status = FLB_SP_ERROR;
-            return -1;
+            return NULL;
         }
     }
 
@@ -180,27 +240,25 @@ int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
     else if (record_func > 0) {
         key->record_func = record_func;
     }
+    else if (timeseries_func > 0) {
+        key->timeseries_func = timeseries_func;
+    }
 
     /* Lookup for any subkeys in the temporal list */
     if (mk_list_size(cmd->tmp_subkeys) > 0) {
-        /* Map context keys into this command key structure */
-        key->subkeys = cmd->tmp_subkeys;
-        cmd->tmp_subkeys = flb_malloc(sizeof(struct mk_list));
-        if (!cmd->tmp_subkeys) {
-            flb_errno();
-            cmd->tmp_subkeys = key->subkeys;
+        ret = swap_tmp_subkeys(&key->subkeys, cmd);
+        if (ret == -1) {
             flb_sp_cmd_key_del(key);
             cmd->status = FLB_SP_ERROR;
-            return -1;
+            return NULL;
         }
-        flb_slist_create(cmd->tmp_subkeys);
 
         /* Compose a name key that include listed sub keys */
         s = flb_sds_len(key->name) + (16 * mk_list_size(key->subkeys));
         key->name_keys = flb_sds_create_size(s);
         if (!key->name_keys) {
             flb_sp_cmd_key_del(key);
-            return -1;
+            return NULL;
         }
 
         tmp = flb_sds_cat(key->name_keys, key->name, flb_sds_len(key->name));
@@ -218,7 +276,7 @@ int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
             }
             else {
                 flb_sp_cmd_key_del(key);
-                return -1;
+                return NULL;
             }
 
             /* selected key name */
@@ -229,7 +287,7 @@ int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
             }
             else {
                 flb_sp_cmd_key_del(key);
-                return -1;
+                return NULL;
             }
 
             /* suffix */
@@ -239,12 +297,27 @@ int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
             }
             else {
                 flb_sp_cmd_key_del(key);
-                return -1;
+                return NULL;
             }
         }
     }
 
+    return key;
+}
+
+int flb_sp_cmd_key_add(struct flb_sp_cmd *cmd, int func,
+                       const char *key_name, const char *key_alias)
+{
+    struct flb_sp_cmd_key *key;
+
+    key = flb_sp_key_create(cmd, func, key_name, key_alias);
+
+    if (!key) {
+        return -1;
+    }
+
     mk_list_add(&key->_head, &cmd->keys);
+
     return 0;
 }
 
@@ -304,7 +377,6 @@ struct flb_sp_cmd *flb_sp_cmd_create(const char *sql)
 
     /* Condition linked list (we use them to free resources) */
     mk_list_init(&cmd->cond_list);
-
     mk_list_init(&cmd->gb_keys);
 
     /* Allocate temporal list and initialize */
@@ -316,6 +388,15 @@ struct flb_sp_cmd *flb_sp_cmd_create(const char *sql)
     }
     flb_slist_create(cmd->tmp_subkeys);
 
+    /* Allocate linked list and initialize */
+    cmd->tmp_params = flb_malloc(sizeof(struct mk_list));
+    if (!cmd->tmp_params) {
+        flb_errno();
+        flb_free(cmd);
+        return NULL;
+    }
+    mk_list_init(cmd->tmp_params);
+
     /* Flex/Bison work */
     yylex_init(&scanner);
     buf = yy_scan_string(sql, scanner);
@@ -409,8 +490,9 @@ const char *flb_sp_cmd_stream_prop_get(struct flb_sp_cmd *cmd, const char *key)
 
 /* WINDOW functions */
 
-void flb_sp_cmd_window(struct flb_sp_cmd *cmd,
-                       int window_type, int size, int time_unit)
+int flb_sp_cmd_window(struct flb_sp_cmd *cmd,
+                      int window_type, int size, int time_unit,
+                      int advance_by_size, int advance_by_time_unit)
 {
     cmd->window.type = window_type;
 
@@ -425,6 +507,26 @@ void flb_sp_cmd_window(struct flb_sp_cmd *cmd,
         cmd->window.size = (time_t) size * 3600;
         break;
     }
+
+    if (window_type == FLB_SP_WINDOW_HOPPING) {
+        switch (advance_by_time_unit) {
+        case FLB_SP_TIME_SECOND:
+            cmd->window.advance_by = (time_t) advance_by_size;
+            break;
+        case FLB_SP_TIME_MINUTE:
+            cmd->window.advance_by = (time_t) advance_by_size * 60;
+            break;
+        case FLB_SP_TIME_HOUR:
+            cmd->window.advance_by = (time_t) advance_by_size * 3600;
+            break;
+        }
+
+        if (cmd->window.advance_by >= cmd->window.size) {
+            return -1;
+        }
+    }
+
+    return 0;
 }
 
 /* WHERE <condition> functions */
@@ -474,6 +576,7 @@ struct flb_exp *flb_sp_cmd_comparison(struct flb_sp_cmd *cmd,
 struct flb_exp *flb_sp_cmd_condition_key(struct flb_sp_cmd *cmd,
                                          const char *identifier)
 {
+    int ret;
     struct flb_exp_key *key;
 
     key = flb_calloc(1, sizeof(struct flb_exp_key));
@@ -486,19 +589,14 @@ struct flb_exp *flb_sp_cmd_condition_key(struct flb_sp_cmd *cmd,
     key->name = flb_sds_create(identifier);
     mk_list_add(&key->_head, &cmd->cond_list);
 
-    /* Lookup for any subkeys in the temporal list */
     if (mk_list_size(cmd->tmp_subkeys) > 0) {
-        key->subkeys = cmd->tmp_subkeys;
-        cmd->tmp_subkeys = flb_malloc(sizeof(struct mk_list));
-        if (!cmd->tmp_subkeys) {
-            flb_errno();
-            cmd->tmp_subkeys = key->subkeys;
+        ret = swap_tmp_subkeys(&key->subkeys, cmd);
+        if (ret == -1) {
             flb_sds_destroy(key->name);
             mk_list_del(&key->_head);
             flb_free(key);
             return NULL;
         }
-        flb_slist_create(cmd->tmp_subkeys);
     }
 
     return (struct flb_exp *) key;
@@ -631,9 +729,10 @@ void flb_sp_cmd_condition_add(struct flb_sp_cmd *cmd, struct flb_exp *e)
 
 int flb_sp_cmd_gb_key_add(struct flb_sp_cmd *cmd, const char *key)
 {
+    int ret;
     struct flb_sp_cmd_gb_key *gb_key;
 
-    gb_key = flb_malloc(sizeof(struct flb_sp_cmd_gb_key));
+    gb_key = flb_calloc(1, sizeof(struct flb_sp_cmd_gb_key));
     if (!gb_key) {
         flb_errno();
         return -1;
@@ -645,7 +744,20 @@ int flb_sp_cmd_gb_key_add(struct flb_sp_cmd *cmd, const char *key)
         return -1;
     }
 
+    gb_key->id = mk_list_size(&cmd->gb_keys);
     mk_list_add(&gb_key->_head, &cmd->gb_keys);
+
+    /* Lookup for any subkeys in the temporal list */
+    if (mk_list_size(cmd->tmp_subkeys) > 0) {
+        ret = swap_tmp_subkeys(&gb_key->subkeys, cmd);
+        if (ret == -1) {
+            flb_sds_destroy(gb_key->name);
+            mk_list_del(&gb_key->_head);
+            flb_free(gb_key);
+            return -1;
+        }
+    }
+
     return 0;
 }
 
@@ -681,3 +793,95 @@ void flb_sp_cmd_condition_del(struct flb_sp_cmd *cmd)
         flb_free(exp);
     }
 }
+
+/* Timeseries functions */
+int flb_sp_cmd_param_add(struct flb_sp_cmd *cmd, int func, struct flb_exp *param)
+{
+    struct flb_exp_param *p;
+    struct flb_exp_key *key;
+
+    if (func > 0) { /* Parameter is a pre-defined function */
+        key = (struct flb_exp_key *) flb_sp_cmd_condition_key(cmd, NULL);
+        if (!key) {
+            return -1;
+        }
+        key->func = func;
+        param = (struct flb_exp *) key;
+    }
+
+    p = flb_calloc(1, sizeof(struct flb_exp_param));
+
+    if (!p) {
+        return -1;
+    }
+
+    p->type = FLB_EXP_PARAM;
+    p->param = param;
+
+    mk_list_add(&p->_head, cmd->tmp_params);
+
+    return 0;
+}
+
+int flb_sp_cmd_timeseries(struct flb_sp_cmd *cmd, char *func, const char *key_alias)
+{
+    int i;
+    char *ts_name;
+    struct mk_list *head;
+    struct mk_list *tmp;
+    struct flb_sp_cmd_key *key;
+    struct flb_exp_param *param;
+    struct flb_exp_timeseries *ts;
+
+    /* Timeseries functions are keys with no name, with functions added */
+    for (i = 0; i < TIMESERIES_FUNCTIONS_SIZE; i++)
+    {
+        ts_name = timeseries_functions[i];
+        if (strcmp(ts_name, func) == 0)
+        {
+            key = flb_sp_key_create(cmd, i + FLB_SP_TIMESERIES_START, NULL, key_alias);
+
+            if (!key) {
+                return -1;
+            }
+
+            ts = flb_calloc(1, sizeof(struct flb_exp_timeseries));
+            if (!ts) {
+                flb_errno();
+                cmd->status = FLB_SP_ERROR;
+                flb_sp_cmd_key_del(key);
+                return -1;
+            }
+            mk_list_init(&ts->params);
+
+            /*
+             * Traverse over temporary key list and add it to timeseries key as
+             * parameters
+             */
+            mk_list_foreach_safe(head, tmp, cmd->tmp_params) {
+                param = mk_list_entry(head, struct flb_exp_param, _head);
+                mk_list_del(&param->_head);
+                mk_list_add(&param->_head, &ts->params);
+            }
+
+            mk_list_init(cmd->tmp_params);
+
+            ts->cb_func_alloc = timeseries_functions_alloc_ptr[i];
+            ts->cb_func_clone = timeseries_functions_clone_ptr[i];
+            ts->cb_func_add = timeseries_functions_add_ptr[i];
+            ts->cb_func_rem = timeseries_functions_rem_ptr[i];
+            ts->cb_func_calc = timeseries_functions_calc_ptr[i];
+            ts->cb_func_destroy = timeseries_functions_destroy_ptr[i];
+
+            cmd->timeseries_num++;
+
+            key->timeseries = ts;
+
+            mk_list_add(&key->_head, &cmd->keys);
+
+            return 0;
+        }
+    }
+
+    return -1;
+}
